/*
 * This code is INCLUDED in the tokenizer.
 * I put it in a separate file because
 * lex code tends to mislead emacs' syntax highlighting rules.
 */

static struct SeshLexerState {
  /* The string we are parsing  */
  char * char_entry_id;
  /* The value we are building. */
  Tcl_Obj *val;
  /* toggle values */
  int inShading;
  int color;
  /* */

  /*
   *
   *  The next two fields are used to deal with the ambiguity of #b : just
   *  after a sign, #b is to be understood as "#" + "b", that is :
   *  "a#b" means the signs "a" and "b" overlapping. On the other hand,
   *  d-#b-..... means "sign d", and then begin shading, etc...
   *
   *  signRead is set to TRUE when a sign is read.
   *  signAlreadyRead is used to reset signRead to FALSE afterwards.
   */
     
  int signRead;
  int signAlreadyRead;
  int state;
  int lineNumber;
} seshLexerState = {
  NULL, /* No value yet */
  NULL, /* No string */
  FALSE, /* Not shaded */
  1, /* black */
  FALSE,
  FALSE,
  INITIAL,
  0
};

/**
   prepare to read string
   in a parser
*/

PUBLIC void
Mdc2lst_init_array_read(char *s)
{
  seshLexerState.char_entry_id= s;
}

PRIVATE int 
do_array_read(char *buf)
{
  int c;
  c= *(seshLexerState.char_entry_id++);
  return (c == 0 ? YY_NULL : (buf[0] = c, 1));
}



PRIVATE void
TLINE();

PRIVATE char *
enleve_plus_a_la_fin(char *s);

static Tcl_Obj
computeShading(char *shading)
{
  int i= 0;
  int val= 0;
  Tcl_Obj *result;
  while (shading[i] == '#') i++;
  while (isdigit(shading[i])) {
    switch (shading[i]) {
    case '1':
      val+= 1;
      break;
    case '2':
      val+= 2;
        break;
    case '3':
      val+= 4;
        break;
    case '4':
      val+= 8;
        break;
    }
  }
  result= Tcl_NewIntObj(val);
}

PRIVATE void
TLINE()
{
  char *s;
  s= yytext;
  while ((s= strchr(s, '\n')))
  {
    seshLexerState.lineNumber++;
    s++;
  }
}

PRIVATE char *
enleve_plus_a_la_fin(char *s)
{
  char *fin;
  fin= s + strlen(s) - 1;
  while ((*fin) != '+') fin--;
  while ((*fin) == '+') fin--;
  fin++;
  *fin= 0;
  return s;
}

/**
 * The classic yywrap function.
 * 
 */

int
yywrap(void)
{
  return 1;
}

/**
 * Initialisation
 */

PUBLIC void 
Mdc2lst_init_seshlex()
{
  struct SeshLexerState defaultLexerState= {
    NULL,
    NULL, /* No string */
    FALSE, /* Not shaded */
    1, /* black */
    FALSE,
    FALSE,
    INITIAL,
    0
  };
  seshLexerState= defaultLexerState;
  yyrestart( yyin );
}

PRIVATE void
setSignRead()
{
  /* We have just read a sign */
  seshLexerState.signRead= TRUE;
  /* This sign hasn't been used yet. */
  seshLexerState.signAlreadyRead= FALSE;
}

PRIVATE int
signWasRead() 
{
  return seshLexerState.signRead;
}

PRIVATE void
updateSignRead()
{
  if (signWasRead())
    if (seshLexerState.signAlreadyRead)
      {
	seshLexerState.signAlreadyRead= FALSE;
	seshLexerState.signRead= FALSE;
      }
  else
    {
      seshLexerState.signAlreadyRead= TRUE;
    }
}

PRIVATE void
setVal(Tcl_Obj *o) {
  seshLexerState.val= o;
}

PRIVATE void
setTextVal(char *s) 
{
  setVal(Tksesh_NewTclKeyword(s));
}

PRIVATE void
setIntVal(int i)
{
  setVal(Tcl_NewIntObj(i));
}
