#ifndef BTREE_H
#define BTREE_H

#include "btdb.h"

typedef unsigned char dbptr[4] ; /*  32 bits databases */

/* size of a block in the btree structure */

#define TREEBLOCK 128 

/* structure of a node for the Btree */

#define NNODES 8

struct btree_node {
	int is_on_disk;								/* is the node saved to the disk ? */
	int should_be_saved;					/* should the node be saved ? */
	int n;												/* number of elements in the node */
	dbptr_t address_on_disk;			/* address of the node on disk */
	pstring elts[NNODES+1];				/* keys for the elements */
	dbptr_t data[NNODES+1];				/* data pointers for the elements */
	dbptr_t ptrs[NNODES+2];				/* pointers to the next nodes */
};

typedef struct btree_node btree_node_t;

/*
 * struct btdb_index : 
 * points to an element in the btree. 
 * Will NOT be valid in case of new insertions and the like.
 *  is the current node (to reduce disk usage)
 * s is a list of btdb_index_elt.
 */

struct btdb_index_elt {
	struct btree_node nd;					/* current node */
	int i;												/* index in node. 
																	 more precisely, 
																	 correspond to the last subtree seen
																	 and/or to the next Element. */
};

struct btdb_index {
	int n;												/* index of last elt in s */
	Chaine s;
	btdb_t * arbre;
};

/* Low level struct 
	 struct block_list {
	 int fd;
	 dbptr_t block_size;
	 dbptr_t first_free;
	 };
*/

enum whatToFind {FIND_LESSER, FIND_GREATER};

#endif


/*
 * Local Variables:
 * tab-width: 2
 * End:
 */

