
#include "general.h"

#include <fcntl.h>

#include "simplefile.h"

#ifdef USEPOSIXFILE

int
MyOpen(struct MyFile *f, char *name, enum MyOpenFlags flags)
{
  int fflag= 0;
  switch (flags) {
  case MYREAD:
    fflag= O_RDONLY|O_BINARY;
    break;
  case MYCREATE:
    fflag= O_WRONLY|O_CREAT|O_TRUNC|O_BINARY;
    break;
  case MYWRITE:
    fflag= O_RDWR|O_BINARY;
    break;    
  }
  
  f->f= open(name, fflag, 0666);
  f->last_was_write= TRUE;
  if (f->f == -1)
    return -1;
  else 
    {
      MySeek(f, 0, SEEK_SET);
      return 0;
    }
}

void
MyClose(struct MyFile *f)
{
  close(f->f);
}

int
MyWrite(struct MyFile *f, void *buf, size_t count)
{
  return write(f->f, buf, sizeof(char) * count);
}

int
MyRead(struct MyFile *f, void *buf, size_t count)
{
  return read(f->f, buf, sizeof(char)* count);
}

void
MySeek(struct MyFile *f, long offset, int whence)
{
  lseek(f->f, offset, whence);
}

int
MyTell(struct MyFile *f)
{
  return lseek(f->f,0, SEEK_CUR);
}

int
MyAccess(char *name, int flag)
{
  return access(name, flag);
}

#else

int
MyOpen(struct MyFile *f, char *name, enum MyOpenFlags flags)
{
  char * fflag;
  switch (flags) {
  case MYREAD:
    fflag= "rb";
    break;
  case MYCREATE:
    fflag= "wb+";
    break;
  case MYWRITE:
    fflag= "rb+";
    break;    
  }
  
  f->f= fopen(name, fflag);
  f->last_was_write= TRUE;
  if (f->f == NULL)
    return -1;
  else 
    {
      MySeek(f, 0, SEEK_SET);
      return 0;
    }
}

void
MyClose(struct MyFile *f)
{
  fclose(f->f);
}

int
MyWrite(struct MyFile *f, void *buf, size_t count)
{
  int res;
  if (! f->last_was_write) {
    MySeek(f, 0, SEEK_CUR);
    f->last_was_write= TRUE;		
  }
  res= fwrite(buf, sizeof(char), count, f->f);
  return res;
}

int
MyRead(struct MyFile *f, void *buf, size_t count)
{
  if (f->last_was_write) {
    MySeek(f, 0, SEEK_CUR);
    f->last_was_write= FALSE;
  }
  return fread(buf, sizeof(char), count, f->f);
}

void
MySeek(struct MyFile *f, long offset, int whence)
{
  long pos;
  switch(whence)
    {
    case SEEK_SET:
      fseek(f->f, offset, whence);
      break;
    case SEEK_END:
      fseek(f->f, 0, SEEK_END);
      pos= ftell(f->f);
      fseek(f->f, pos+offset, SEEK_SET);
      break;
    case SEEK_CUR:
      pos= ftell(f->f);
      fseek(f->f, offset+pos, SEEK_SET);
      break;
    }
}

int
MyTell(struct MyFile *f)
{
  return ftell(f->f);
}

int
MyAccess(char *name, int flag)
{
  #ifdef MAC_TCL
  return TclAccess(name, flag);
  #else
  return access(name,flag);
  #endif
}
#endif
