/*
 * Heading for the tkwin drawing device for tksesh
 * This device allows one to draw on a tk widget.
 */

/*  1998, 2000 Serge Rosmorduc,
 * quipe EC.art, Laboratoire LRIA, Universit de Paris 8
 *
 * 15 rue Catulienne
 * 93236 St Denis Cedex
 * FRANCE
 *
 * Tout droits rservs.
 * 
 * voyez le fichier ../LICENCE.txt pour plus d'informations
 * See the ../LICENCEE.txt file for more information
 *
 * $Id: TKSO_tkwin.h,v 1.1.1.1 2002/04/16 13:55:00 rosmord Exp $
 *
 */

/*
 * $Log: TKSO_tkwin.h,v $
 * Revision 1.1.1.1  2002/04/16 13:55:00  rosmord
 * Imported using tkCVS
 *
 * Revision 1.1  2000/08/04 11:52:04  rosmord
 * Initial revision
 *
 */

#ifndef TKSO_TKWIN_H
#define TKSO_TKWIN_H

#include "tgraph.h"
#include <tk.h>

typedef struct TKSESH_palette_S {
	int complete;
	XColor **colors;
	int ncolors;									/* number of colors */
	GC * gcs;
	/*** special gcs : ***/
	GC erasegc, copygc, graygc, invertgc, textgc;
	Tk_Font fonttouse;						/*  font for writing in windows*/
} TKSESH_palette;

typedef struct TKSO_tkwin_struct {
	TKSESH_output header;
	Display *display;
	Drawable d;
	Tk_Window win;
	Tcl_Interp *interp;
	GC gctouse;
	TKSESH_palette *p;
	int is_attached;										// is this output attached to an existing window ?
} TKSO_tkwin;

/* 
 * TKSO_tkwin_init --
 * Creates a new output corresponding to a tk window 
*/

TKSO_tkwin *
TKSO_tkwin_init(Display *d,									 
								double x1, double y1,
								double x2, double y2,
								int w, int h,
								Tk_Window win,
								Tcl_Interp *interp);

/*
 * frees the output 
 * Only deal with memory stuff and the like. Graphical ressources
 * are dealt with in another way.
 */

void
TKSO_tkwin_free(TKSO_tkwin *o);

/*
 * Associates a palette with an output
 */

void
TKSO_tkwin_set_palette(TKSO_tkwin *o, TKSESH_palette * palette);

/*
 * Frees the graphic ressource and memory associated with palette.
 */

void
TKSO_tkwin_free_palette(Display *d, TKSESH_palette* p);

/*
 *  The drawing space associated with a window is not 
 *  available in the beginning. So, we must call init_draw to
 *  complete the TKSO_tkwin
 */

void
TKSO_tkwin_init_draw(TKSO_tkwin *o);

/*
 * frees the drawable associated with o, if 
 * possible.
 * If the said drawable is managed by some other system,
 * it won't be freed.
 */
void
TKSO_tkwin_free_draw(TKSO_tkwin *o);

/*
 * Ok. Now, there are a number of copy functions.
 */

/*
 * Copy the graphic part of a in b.
 * Does nothing else. It's really for cases where a and b are the same size
 * and all
 */

void
TKSO_tkwin_copy(TKSO_tkwin *a, TKSO_tkwin *b);

/*
 * Copy a part from depart into arrivee. The part is described by
 * fenetre, and the target point is destination
 */

void
TKSO_tkwin_trans_copie_fenetre(TKSO_tkwin *depart,
															 TKSO_tkwin *arrivee,
															 TKSESH_rectangle fenetre,
															 TKSESH_point destination);

/*
 * Makes o a TKSO_tkwin the same size as orig,
 * but with a blank drawing surface.
 */

int
TKSO_tkwin_get_working_copy(TKSO_tkwin *o, 
														TKSO_tkwin *orig);
/*
 * Makes o a copy of orig,
 * Stealing the same graphical resources
 */

void
TKSO_tkwin_steal_copy(TKSO_tkwin *o, 
											TKSO_tkwin *orig);

/*
 * Return true if we can draw to o
 */

int
TKSO_tkwin_is_ready(TKSO_tkwin *o);

/*
 * Now, This one does the same thing as TKSO_tkwin_get_working_copy,
 * but instead of allocating new resources, it first tries to steal them from old
 */

int
TKSO_tkwin_recycle(TKSO_tkwin *newout, TKSO_tkwin *orig, TKSO_tkwin *old);

XImage *
TKSO_tkwin_getImage(TKSO_tkwin *);

int
TKSO_tkwin_hasPalette(TKSO_tkwin * o);

TKSESH_palette *
TKSO_tkwin_new_palette(int colc,
											 XColor **colv);

//struct TKSO_tkwin {
//}

#endif

/* Let's preserve the present presentation
 * Local Variables:
 * tab-width: 2
 * c-block-comments-indent-p: nil
 * End:
 */
