/***************************************************************************\				   
								 Definition des quadras a` bas niveau
\***************************************************************************/
/* 	$Id: quadra.h,v 1.1.1.1 2002/04/16 13:55:00 rosmord Exp $	       */
/* from v 1.20 onward, we are moving to a much more "Manuel"-compliant
	 system.  In the spirit of the manuel, parenthesized systems are
	 more often than not treated as "toggle" system.  This was already
	 done for colour and hatchures, but we need to generalize it to
	 cartouches and philological annotations. */
	 
#ifndef _QUADRA_H
#define _QUADRA_H

#include "general.h"
#include "bfont.h"

/*
 * Access to fields in structures */

/* general */
#define S_TYPE_OF(x)	((x).type)
/* #define S_COULEUR_OF(x)	((x).couleur) */
#define S_NEXT_OF(x)    ((x).next)
#define S_PREVIOUS_OF(x)    ((x).previous)
#define S_LAST_OF(x)    ((x).last)
#define S_KERNING_OF(x)      ((x).kerning)

/* Signe */

#define S_TYPE_HACHURE_OF(x)	((x).typehachure)
#define S_EST_FIN_MOT(x)	((x).estfinmot)
#define S_EST_GRAMM(x)	((x).estgram)
#define S_EST_SELECTED(x)	((x).selected)
#define S_SIZE_OF(x)          ((x).size)
#define S_ROTATION_OF(x)      ((x).rotation) 
#define S_REVERT_OF(x)      ((x).revert)
#define S_IGNORE_OF(x)      ((x).ignore) 
#define S_LONG_OF(x)      ((x).islong) 
#define S_TALL_OF(x)      ((x).istall) 
#define S_HIEROGLYPHE_OF(x)   ((x).donnees.hieroglyphe)
#define S_CARACTERE_OF(x)   ((x).donnees.hieroglyphe.caractere)
#define S_LIG_NUMSIGN_OF(x) ((x).donnees.ligature->nbsign)
#define S_LIGW_OF(x)   ((x).donnees.ligature->w)
#define S_LIGH_OF(x)   ((x).donnees.ligature->h)
#define S_FONTE_OF(x)   ((x).donnees.hieroglyphe.lettre)
#define S_CHAR_OF(x)   ((x).donnees.hieroglyphe.l)
#define S_REPRESENTATION_OF(x)   ((x).donnees.hieroglyphe.rep)
#define S_FIRST_SIGN_OF(x)       ((x).donnees.over[0])
#define S_SECOND_SIGN_OF(x)       ((x).donnees.over[1])


#define S_TEXTE_OF(x)         ((x).donnees.texte)

/* Vertlist */
#define VL_QUADRA_OF(x)       ((x).un_quadra)


/* Quadra */
#define QD_SEPARATEUR_OF(x)	  ((x).separateur)
#define QD_VERTLIST_OF(x)     ((x).contenu.horizontal)
#define QD_LIG_OF(x)     ((x).contenu.lig)
#define QD_SIGNE_OF(x)        ((x).contenu.un_signe)
#define QD_QUADRA_OF(x)       ((x).contenu.des_quadras)
#define QD_TAILLE_OF(x)       ((x).contenu.taille)

#define HACHURE1 (1)
#define HACHURE2 (1<<2)
#define HACHURE3 (1<<3)
#define HACHURE4 (1<<4)
#define NOHACHURE 0
#define TOUTHACHURE (HACHURE4|HACHURE3|HACHURE2|HACHURE1)

typedef unsigned char Hachures;

typedef enum _Typehachure 
{
	S_aucune,
	S_ga,
	S_ha,
	S_va, 
	S_ta
} Typehachure;

typedef enum _Typeseparateur 
{
	S_espace,
	S_finligne,
	S_finpage,
	S_intersignes,
	S_sansseparateur
} Typeseparateur;


/* Il y a deux sortes de donne'es inse're'es : 
	 Celles qui parenthe`sent d'autres constructions 
	 Les autres
	 */

typedef enum _Typesigne 
{
	S_hieroglyphe,
	S_overwrite,
	S_inconnu,
	S_romain,
	S_textesuper,
	S_pointrouge,
	S_pointnoir,
	S_haplographie,
	S_gauchedroite, 
	S_droitegauche ,
	S_lacune ,
	S_lignelacune ,
	S_point,
	S_pointpoint,
	S_hachureg,
	S_hachureh,
	S_hachurev,
	S_hachuret
} Typesigne;

typedef enum _Typegroupe
{
	S_signe,											/* feuille */
	S_quadra,											/* quadra  */
	S_ligature,											/* ligature  */
 	S_carto ,											/* cartouche autour du groupe */
	S_debcarto,										/* diverses boites */
	S_rightleft,
	S_milcarto,
	S_fincarto,
	S_serekh,
	S_enceinte,
	S_chateau,
	S_debserekh,
	S_milserekh,
	S_finserekh,
	S_debenceinte,
	S_milenceinte,
	S_finenceinte,
	S_debchateau,
	S_milchateau,
	S_finchateau,
	S_superfetatoire,							/* Les parentheses */
	S_efface,
	S_disparu,
	S_ajoutscribe,
	S_ajoutauteur
}  Typegroupe;

typedef enum {
	Psize, Protation, Phachure, Prevert, Pword, Pgram, Pselect, Pignore, Ptall, Plong
} Signproperty;

/* union Donnee should move and be INDEPENDANT from quadra. 
	 (probably move to signs.c)
	 At least for the hieroglyphestruct part.
	 yeah !*/

typedef struct _Signe {
  Typesigne type;
  Typehachure typehachure;
  int size, rotation, revert;
	int ignore;										/* Should the sign be ignored ? */
	/*
	 * For the two value below, the choice may be left to the system.
	 * Hence we have three values : -1 = system choice, 0 = no, 1 = yes. Default : system choice
	 *
	 */
	int islong;										/* is the sign a long narrow sign which should be streched if needed ? */
	int istall;										/* is the sign a tall narrow sign which should be streched if needed ? */
  int estfinmot;
  int estgram;
  int selected;									/* temporary system ! */
	/* TagList *tags;								 Tags attachs  ce signe */
  union _Donnees {
    struct _hieroglyphestruct {
      char * rep;								/* representation du signe (statique) */
			Lettre *l;								/* le caractre lui-meme */
    } hieroglyphe;
		struct _Signe *over[2];			/* for things like A##a */
    char * texte;								/* symboles divers directement repre'sente's */
  } donnees;
} Signe;


struct Liggroup {
	struct Ligature *l;						/* 
																 * description of how 
																 * things should be presented.
																 * if NULL, it's up to the program.
																 */
	int nbr ;											/* number of signs in ligature */
	struct _Signinlig *signs;			/* list of the signs */
};

/* Structure grouping signs in a "possible" ligature */

struct _Signinlig {
	int nbr;											/* number of signs including this one */
	struct _Signe *sign;
	struct _Signinlig *next;
};

typedef struct _Vertlist {
	/* This part is only meaningful for the first element.
		 We should in fact cut the list in two parts : header and element
	*/
	double scale;									/* scale to use for this (possibly embedded cadrat) */
	double hcompact;							/* "internal" height of the Vertlist WITHOUT spaces */ 
	double w, h;									/* width and height */
	int n;												/* number of lines */
	
	struct _Quadra *un_quadra; /* premie`re ligne */
	int kerning;		/* kerning vertical */	
	struct _Vertlist * next, * previous, *last; /* les suivantes */
} Vertlist;

typedef struct _Quadra {
	Typegroupe type;

	/* This part is only meaningful for the first element.
		 We should in fact cut the list in two parts : header and element
	*/

	/* Plus, there should be a set of dimensions for line and a set for
	 * column setting
	 */

	double scale;
	double h, w;								/* dimensions in horizontal setting */
	
	double wcompact;							/* "internal" width of the cadrat WITHOUT spaces */
	int n;												/* number of groups in the line */

	int kerning;		/* kerning horizontal */
	Typeseparateur separateur; /* separation d avec le signe suivant */
	Hachures hachures;	/* hachures individuelles */
	int est_grise;		/* fait parti d'un texte hachure */
				/* enum _Quadra_couleur {noir, rouge} couleur; */
	union {
		Vertlist  *horizontal; /* liste des lignes du quadra s'il y a lieu */
		Signe * un_signe;
		int taille;	/* la taille des sauts (de ligne ou de page) */
		struct _Quadra * des_quadras; /* quadra imbrique' ! */
		struct Liggroup *lig;						/* a ligature */
	} contenu;
  struct _Quadra * next, *previous, *last;
} Quadra;


typedef struct _Listesignes {		/* Liste des signes d'un cadrat */
	Signe * signe ;								/* pointeur vers le tableau des signes */
	struct _Listesignes * next;
	struct _Listesignes * last;	
} Listesignes;

/*>>>>quadra.c*/

/**
 * PUBLIC void 
 * initialise_quadras()
 * 
 * Should be called once.
 * initialise memory for quadras and such
 * The call is mandatory
 */

PUBLIC void 
initialise_quadras();

/**
 * PUBLIC void 
 * detruit_quadras()
 * 
 * Definitively frees memory from quadras & co.
 * 
 * to be called once in the end
 */

PUBLIC void 
detruit_quadras();

/**
 * Declares all space used for quadras and the like
 * as free space for new quadras
 * 
 * Useful if you want 
 *   1) to destroy current quadras
 *   2) to create some new one later
 *
 * can be called any number of times.
 */

PUBLIC void 
libere_quadras();

/**
 * PUBLIC Quadra *
 * new_quadra(Typegroupe val)
 * 
 * Creates a new quadra of type val
 */

PUBLIC Quadra *
new_quadra(Typegroupe val);

/**
 *
 * PUBLIC Quadra* 
 * new_sign(Typesigne s)
 *
 * Creates a new SIGN of type s
 *
 */

PUBLIC struct _Signe* 
new_sign(Typesigne s);

/**
 * PUBLIC void 
 * free_sign()
 *
 */
PUBLIC void
free_sign(struct _Signe *s);

/**
 * PUBLIC Vertlist * 
 * new_vert()
 *
 * Creates and initialise a new Vertlist
 */

PUBLIC Vertlist * 
new_vert();

PUBLIC struct _Signinlig *
new_signinlig();

PUBLIC struct Liggroup *
new_liggroup();

/**
 * PUBLIC void 
 * detruit_quadra(Quadra *q)
 *
 * Frees a quadra
 */

PUBLIC void 
detruit_quadra(Quadra *q);

/**
 * PUBLIC void 
 * detruit_vertlist(Vertlist *hl)
 * Frees an Vertlist
 *
 */

PUBLIC void 
detruit_vertlist(Vertlist *hl);

/**
 * PUBLIC Listesignes **
 * SESH_creer_listesignes(Quadra *q,
 *                        Listesignes **l)
 *
 * writes in *l a pointer to a signlist
 * linking the signs found in q.
 * 
 */

PUBLIC Listesignes **
SESH_creer_listesignes(Quadra *q,
											 Listesignes **l);

/**
 *PUBLIC void
 *SESH_detruire_listesignes(Listesignes *l)
 *
 * Destroys a signlist. 
 * it DOESN'T set l to NULL.
 *
 */

PUBLIC void
SESH_detruire_listesignes(Listesignes *l);

/**
 *PUBLIC int
 *SESH_length_listesignes(Listesignes *l)
 *
 * returns the number of signs in a signlist.
 */

PUBLIC int
SESH_length_listesignes(Listesignes *l);

/**
 *	PUBLIC Listesignes *
 *	SESH_listesignes_nth(Listesignes *l, int n)
 *  
 *  returns the nth sign of a signlist, or NULL if
 *  there is no such sign.
 *
 *  the 1st signe is numbered 1
 *
 *  The pointer points to the actual sign in the quadra,
 *  which allows direct manipulation 
 *  (e.g. changing some flags)
 */

PUBLIC Listesignes *
SESH_listesignes_nth(Listesignes *l, int n);

/*<<<<*/

/*>>>>quad2text.c */

/**
 * PUBLIC char *
 * quad2text(Quadra *q)
 *
 * Sends back a string in MDC format representing q
 *	
 */

PUBLIC char *
quad2text(Quadra *q, int breakit, int red, int shaded);

/*<<<<*/


#endif


/* Let's preserve the present presentation
 * Local Variables:
 * tab-width: 2
 * c-block-comments-indent-p: nil
 * End:
 */
