#ifndef TKS_GEOMETRY
#define TKS_GEOMETRY

/*
 * The following structure gives Geometry information
 * to use when drawing a hieroglyphic text.
 */

typedef struct _TKSTexteGeometry {
  double natural_h;	/* height of a cadrat according to the fonts
			 * (doesn't correspond to a scale). Basically, 
			 * it's a constant.
			 */
  double natural_w;
  double maxh, maxw;	/* basic "normal" Dimensions for a cadrat */
  double smallskip;     /* small skip between subcadrats */
  int show_words;
  int show_gram;
  int vcenter;
  int right_left;
  int in_columns;
} TKSTexteGeometry;


void
TKSGeom_init(TKSTexteGeometry *geom);

/* "Natural" dimensions : 
   Any sign with dimensions greater than this one will be downscaled.
   Other signs will get a 1.0 scale
*/

double
TKSGeom_getNaturalHeight(TKSTexteGeometry *geom);

double
TKSGeom_getNaturalWidth(TKSTexteGeometry *geom);

/*
 * Maximal size of a cadrat. Relative to Natural size.
 */

double
TKSGeom_getMaxH(TKSTexteGeometry *geom);

double
TKSGeom_getMaxW(TKSTexteGeometry *geom);

double
TKSGeom_getSmallSkip(TKSTexteGeometry *geom);

int
TKSGeom_getShowWords(TKSTexteGeometry *geom);

int
TKSGeom_getShowGram(TKSTexteGeometry *geom);

void
TKSGeom_setCentered(TKSTexteGeometry *geom, int val);

int
TKSGeom_isCenteredP(TKSTexteGeometry *geom);

int
TKSGeom_isRightLeftP(TKSTexteGeometry *geom);

int
TKSGeom_isInColumnsP(TKSTexteGeometry *geom);

#endif
