/*
	 Various routines used by the system, mainly
	 to implement the actions
*/
#ifndef TKSESHAUX_H
#define TKSESHAUX_H

/*>>>>tkseshaux.c  */

/**
 *
 * Marks are names for places in texts.
 *
 * They can be set and retrieved and are used
 * as symbolic names to manipulate text
 *
 * A number of names are reserved for special marks :
 * 
 *     "insert" is the name of the "current" point in the text.
 *     "ecran"  is the name of the beginning of the text that 
 *              appears on the screen.
 *
 * 
 * Sends back a pointer to the Texte 
 * item marked with m
 *
 */

PUBLIC GeneralIndex
SESH_get_marque(Sesh *hiero, char *m);


/**
 * PUBLIC void
 * SESH_detruire_marques(Sesh *hiero)
 * 
 * Suppresses all marks
 */

PUBLIC void
SESH_detruire_marques(Sesh *hiero);

/**
 * PUBLIC void
 * SESH_detruire_marque(Sesh *hiero, char *m)
 * 
 * Suppresses the mark m
 */

PUBLIC void
SESH_detruire_marque(Sesh *hiero, char *m);

/**
 *
 * PUBLIC void
 * SESH_set_marque(Sesh *hiero, char *m, GeneralIndex ou)
 *
 * Sets a mark to a given Text item.
 * 
 */

PUBLIC void
SESH_set_marque(Sesh *hiero, char *m, GeneralIndex ou);

/**
 * INDEXES :
 * 
 * Indexes are manipulated through textual description.
 * They are mostly compatible with the indexes of text widgets
 * 
 * An index is the description of a place in the text.
 * The general form of indexes are :
 * 
 * PLACE MODIFIER
 * 
 * where PLACE is one of :
 *
 *   insert => the insertion point
 *   /DESCRIPTION/ => the textual description of a place in the text,
 *                    according to the description system used for it.
 *                    e.g. could be :
 *                    "colonne 1, 3"
 *                    or 
 *                    "page 5, ligne 7"
 *                    or
 *                    "mur nord, colonne 5, 7"
 *                    ...
 *
 *   LINE.QUADNUMBER => absolute coordinates
 *   @X,Y => screen coordinate (mouse position-compatible)
 *   end => end of text 
 *   TAG.first => beginning of TAG // NOT IMPLEMENTED
 *   TAG.end   => end of TAG // NOT IMPLEMENTED
 *
 *   MARK => the place marked with MARK
 * 
 * And MODIFIER is either nothing or :
 * 
 * ('+'|'-') NUMBER ('quad'|'line')
 * ('linestart'|'lineend')
 * ('wordstart'|'wordend')
 * 
 * '.' SIGNNUMBER
 *
 * PUBLIC GeneralIndex
 * SESH_index(Sesh *hiero, int argc, char **argv, int *pos)
 *
 * 
 * Sends back a pointer to a textual item
 * from a textual index description.
 */

PUBLIC GeneralIndex
SESH_index(Sesh *hiero, int argc, char **argv, int *pos);

/**
 * PUBLIC void
 * SESH_trouve_index(Sesh *hiero, GeneralIndex g, char* index)
 *
 * sends back in char * index the index of the GeneralIndex g
 * in the form LINE.CADRAT
 *
 */

PUBLIC void
SESH_trouve_index(Sesh *hiero, GeneralIndex g, char* index);

/**
 *
 *
 * SESH_compare_indexes
 *
 * takes two indexes i1 and i2, returns 0
 * if they are equal,
 * a positive number if i1 > i2 (i1 after i2)
 * a negative one if i1 < i2
 *
 */

PUBLIC int
SESH_compare_indexes(Sesh *hiero, GeneralIndex i1, GeneralIndex i2);

/**
 *
 *
 * The returned value is the absolute textual description of the
 * location 't' according to the coordinate system set for the given
 * text.
 *
 */

PUBLIC char *
SESH_trouve_index_texte(Sesh *hiero, GeneralIndex g);

/**
 * PUBLIC void
 * SESH_largeur(Sesh *hiero, double *avant, double *apres)
 *
 * For scrollbars :
 * 
 * sets in avant and apres the percentage of horizontal 
 * text shown
 */

PUBLIC void
SESH_largeur(Sesh *hiero, double *avant, double *apres);

/**
 * PUBLIC void
 * SESH_hauteur(Sesh *hiero, double *avant, double *apres)
 *
 * For scrollbars :
 * 
 * sets in avant and apres the percentage of vertical
 * text shown
 */

PUBLIC void
SESH_hauteur(Sesh *hiero, double *avant, double *apres);

/**
 *
 * Sesh_XUpdate --
 *
 *	Update the Horizontal scrollbar in case the display window X view change
 *  (using xScrollCmd)
 *  
 */

PUBLIC void
SESH_XUpdate(Sesh *SeshPtr);

/**
 *
 * Sesh_YUpdate --
 *
 *	Update the vertical scrollbar in case the display window Y view change
 *  (calls yScrollCmd)
 *
 */

PUBLIC void
SESH_YUpdate(Sesh *SeshPtr);

/**
 * PUBLIC void
 * SESH_get_index_position(Sesh* hiero, Texte *debut, int * x, int * y)
 *
 * Sends back the screen position of a textual item in x and y
 * for use with mouse and the like.
 * 
 */

PUBLIC void
SESH_get_index_position(Sesh* hiero, GeneralIndex gdebut, int * x, int * y);

/**
 *
 * computes the length of the current line
 * (in pixels)
 * AS a side effect, all x cadrats coordinates will be accurate
 * *t ends being a pointer to the end of the line. 
 */

PUBLIC double
SESH_compute_line_length(Sesh *hiero, Texte **t);

/**
 * Returns the width and height for a rectangle including the selection.
 *
 * PUBLIC void
 * SESH_compute_sel_dimensions(Sesh *hiero, GeneralIndex deb, GeneralIndex end, double *w, double *h)
 */

void
SESH_compute_sel_dimensions(Sesh *hiero, GeneralIndex gdebut, GeneralIndex gfin, double *w, double *h);

/**
 * PUBLIC int
 * SESH_max_x_dim(Sesh *hiero)
 *
 * Sends back the maximum x dimension of text.
 */

PUBLIC int
SESH_max_x_dim(Sesh *hiero);

/**
 * PUBLIC int
 * SESH_max_y_dim(Sesh *hiero)
 *
 * Sends back the maximum y dimension of text.
 */

PUBLIC int
SESH_max_y_dim(Sesh *hiero);

/**
 * 
 * Sends back the line number (from the beginning of the text)
 * of GeneralIndex ghere
 */

PUBLIC int
SESH_line_number(Sesh *hiero, GeneralIndex ghere);

/**
 * int
 * SESH_jump_of(Sesh * hiero, Texte *t)
 *
 * Used to know how many lines are spanned by a textual item.
 */

PUBLIC int
SESH_jump_of(Sesh * hiero, Texte *t);

/**
	RMRQ IMPORTANTE : void MDCTexte pour la dfinition des zones.
*/

/**
   get the number of lines from gt1 to gt2, gt1 being the first screen
   position
*/

PUBLIC int
SESH_get_screen_pos(Sesh *hiero, GeneralIndex gt1, GeneralIndex gt2);

/**
   get the number of lines between gt1 and gt2
 */

PUBLIC int 
SESH_get_number_of_lines(Sesh *hiero, GeneralIndex gt1, GeneralIndex gt2);

/**
 *
 * sends back a pointer to a Texte deltaN lines from "depart"
 *
 */

PUBLIC GeneralIndex
SESH_moveby_lines(Sesh * hiero, GeneralIndex depart, int deltaN);

/**
 *
 * sends back a pointer to a Texte deltaN lines from "depart",
 * counting lines as displayed on the screen.
 *
 */

PUBLIC GeneralIndex
SESH_moveby_screenlines(Sesh * hiero, GeneralIndex depart, int deltaN);

/**
 * Texte *
 * SESH_find_screen_position(Texte * depart, TKSESH_point p)
 * 
 * given a point with REAL coordinates (NOT INTEGER),
 * sends back a pointer to the corresponding text.
 *
 */

PUBLIC Texte *
SESH_find_screen_position(Sesh * hiero, Texte * depart, TKSESH_point p);

/**
 * PUBLIC int
 * SESH_goto_previous_line(Sesh *hiero, Texte *here, Texte ** but)
 * 
 * goes to the previous line. 
 * As a line might span more than one screen line, sends back the
 * number of spanned lines
 * 
 */

PUBLIC int
SESH_goto_previous_line(Sesh *hiero, Texte *here, Texte ** but);

/**
 * PUBLIC int
 * SESH_goto_next_line(Sesh *hiero, Texte *here, Texte ** but)
 * 
 * goes to the next line. 
 * As a line might span more than one screen line, sends back the
 * number of spanned lines
 * 
 */
	
PUBLIC int
SESH_goto_next_line(Sesh* hiero, Texte *here, Texte **but);

/**
 * Sends a pointer to the beginning of the screen
 */

PUBLIC Texte *
SESH_start_of_screen(Sesh * hiero);

/**
 * Increments the counter which is used to know if current cadrats are
 * up to date
 */

PUBLIC void
SESH_increment_compteur(Sesh *hiero);

/*<<<<*/


#endif
