#ifndef TKSESH_H
#define TKSESH_H

#include "tk.h"
#include "bfont.h"
#include "tgraph.h"
#include "TKSO_tkwin.h"
#include "mdctexte.h"
#include "mdccoord.h"
#include "tks_geometry.h"

/*
 * A data structure of the following type is kept for each square
 * widget managed by this file:
 */


#define TKSESHNBCOLORS 8

typedef struct _Sesh
{
	Tk_Window tkwin;
	Display *display;
	Tcl_Interp *interp;
	Tcl_Command widgetCmd;
	
	/* 
	 * There be Thyngues special to our widget
	 */
	TKSO_tkwin *affichage; /* current screen */
	TKSO_tkwin *old_affichage; /* previous screen, for double buffering */
	TKSO_tkwin *past_affichage; /* the previous old_affichage.
																		We keep it for possible re-use. X performs ok without
																		this, but maybe it makes things faster on the Mac */
	int past_affichage_is_usable;		/* is past affichage usable  ? */
	int keep_old;									/* Should we keep old pixmaps for reuse ?*/
	int borderWidth;							/* Width of 3-D border around whole widget. */
	Tk_3DBorder bgBorder;					/* Used for drawing background. */
	XColor *foreground;						/* Foreground color */
	XColor *redcolor;							/* used for "red texts" */
	XColor *graycolor;						/* used for "ignored text" */
	XColor *gramcolor;						/* used for grammatical endings */
	XColor *gramendcolor;					/* used for grammatical endings 
																 *  + word endings */	
	XColor *cursorcolor;					/* guess what ? */
	TKSESH_palette * palette;

	int relief;										/* Indicates whether window as a whole is
																 * raised, sunken, or flat. */
	int updatePending;						/* Non-zero means an update 
																 * has already been scheduled. */
	/* for scrollbars */
	char * xScrollCmd;
	char * yScrollCmd;
	
	/* DATA ON TEXT ITSELF */

	Texte * debut_texte;					/* INDESTRUCTIBLE */
	Texte * old_debut_ecran;			/* DESTRUCTIBLE => a traiter */
	Texte * montexte;							/* DESTRUCTIBLE ET TRAITE */
	Marque * les_marques;
	
	/* The following values are directly changed through configure */
	int htline;										/* hauteur TOTALE d'une ligne 
																 * (y compris linesep)*/
	int linesep;									/* separation inter-ligne */
	double origx;									/* origine du coin en haut  gauche */

	int largeur;									/* largeur *demande* */
	int hauteur;									/* hauteur *demande* */
	
	/* These values are computed and cached */
	
	int nombre_de_lignes_affichees;	/* nombre de lignes affiches */
	int max_lignes;								/* nombre de lignes de texte */
	int old_max_lignes;						/* ancienne valeur (pour dtecter les
																 * changements) */
	double virtuallinesep;				/* line separation on the virtual screen */
	int old_w;										/* previous *actual* width */
	int old_h;										/* previous *actual* height */
	double maxx;									/* maximal width of an existing line 
																 *                    (vs. displayed)*/
	double old_maxx;
	double echelle;								/* unit.pixel^-1. Mostly used to avoid dividing by htline */
	double resolution;						/* real height of a quadra, in units (internal use)*/
																/* (still to avoid calculating in on the fly) */
	double petit_espace;					/* extra space, in units (internal use) */
	unsigned long int compteur;		/* used to know if a quadra is up-to-date
																 * (timestamp) */
	unsigned long int precedent_compteur;

	/* state variables */
	TKSTexteGeometry geom;
	/* int right_to_left;						0 is left-to-right, 1 is right-to-left. */
	/* int show_words;								 display word frontiers ? */
	/* int in_columns; 0 means lines, 1 means columns  */
	/* int vertical_centering; small signs are centered  */

	int draw_cursor;							/* display blue cursor at insert point ? */
	int display_comments;					/* display text comments  ? */
	int takefocus;								/* takes keyboard focus  ? */	
	int wrap;											/* wrap text ? */
	int display_needed;						/* internal use : do we need to redraw
																 * the pannel ? */
	Coordinate_System coordinates; /* text coordinates */
} Sesh;

extern void 
SeshMyInit(Tcl_Interp *interp);

extern 
Tcl_CmdProc SeshCmd ;

#endif

/* Let's preserve the present presentation
 * Local Variables:
 * tab-width: 2
 * c-block-comments-indent-p: nil
 * End:
 */
