/*
  New type for canvas :
  bezier curves.
  */

#include <tk.h>

typedef struct SplineItem {
  Tk_Item header;		/* x1, y1, x2, 
				 * and y2 => bbox integer 
				 *           canvas coordinates */
  double x, y;
  double x1, y1;
  Tk_Canvas canvas;
  XColor *fgColor;
  XColor *bgColor;
  GC gc;
} SplineItem;

/*
  The structure must be in permanent memory (either
  statically  allocated  or  dynamically allocated but never
  freed);  Tk retains a pointer to this structure.
  */


Tk_ItemType tkSplineType = {
    "spline",			/* name */
    sizeof(SplineItem),		/* itemSize */
    SplineCreate,		/* createProc */
    configSpecs,		/* configSpecs */
    SplineConfigure,		/* configureProc */
    SpLineCoords,		/* coordProc */
    SplineDelete,		/* deleteProc */
    SplineDisplay,		/* displayProc */
    0,					/* alwaysRedraw */
    SplineToPoint,			/* pointProc */
    SplineToArea,				/* areaProc */
    SplineToPostscript,			/* postscriptProc */
    SplineScale,				/* scaleProc */
    SplineTranslate,			/* translateProc */
    (Tk_ItemIndexProc *) NULL,		/* indexProc */
    (Tk_ItemCursorProc *) NULL,		/* icursorProc */
    (Tk_ItemSelectionProc *) NULL,	/* selectionProc */
    (Tk_ItemInsertProc *) NULL,		/* insertProc */
    (Tk_ItemDCharsProc *) NULL,		/* dTextProc */
    (Tk_ItemType *) NULL		/* nextPtr */
};


/* typedef int	Tk_ItemCreateProc _ANSI_ARGS_((Tcl_Interp *interp, */
/* 		    Tk_Canvas canvas, Tk_Item *itemPtr, int argc, */
/* 		    char **argv)); */

static int
SplineCreate(Tcl_Interp *interp, Tk_Canvas canvas, 
		 Tk_Item *itemPtr, int argc, 
		 char **argv, int flags)
{
  SplineItem *s = (SplineItem *) itemPtr;
  
  if (argc < 4) {
    Tcl_AppendResult(interp, "wrong # args: should be \"",
		Tk_PathName(Tk_CanvasTkwin(canvas)), " create ",
		     itemPtr->typePtr->name, " x1 y1 x2 y2 ?x3 y3 ...? ?options?\"",
		     (char *) NULL);
    return TCL_ERROR;
    }

  s->canvas= canvas;

  if (Splinecoords(interp, canvas, itemPtr, i, argv) != TCL_OK) {
    DeleteLine(canvas, itemPtr, Tk_Display(Tk_CanvasTkwin(canvas)));
    return TCL_ERROR;
  }

  if (SplineConfigure(interp, canvas, itemPtr, argc-i, argv+i, 0) == TCL_OK) {
    return TCL_OK;
  }
}

static int 
SplineConfigure(Tcl_Interp *interp, 
		Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
		char **argv, int flags)
{
  SplineItem *linePtr = (LineItem *) itemPtr;
  
}

static int SplineCoord(Tcl_Interp *interp, 
			  Tk_Canvas canvas, Tk_Item *itemPtr, 
			  int argc, char **argv);

static void
SplineDelete(Tk_Canvas canvas, Tk_Item *itemPtr, 
	     Display *display);

static void 
SplineDisplay(Tk_Canvas canvas, 
	      Tk_Item *itemPtr, Display *display, 
	      Drawable dst, 
	      int x, int y, int width, int height);

static double
SplinePoint(Tk_Canvas canvas, Tk_Item *itemPtr,
	    double *pointPtr);

static int
SplineAreaProc(Tk_Canvas canvas, 
		  Tk_Item *itemPtr, double *rectPtr); 

static int
SplinePostscriptProc(Tcl_Interp *interp, 
		     Tk_Canvas canvas, Tk_Item *itemPtr,
		     int prepass);
 
static void	
SplineScaleProc(Tk_Canvas canvas, 
		Tk_Item *itemPtr, 
		double originX, double originY, 
		double scaleX, double scaleY); 

static void
SplineTranslateProc(Tk_Canvas canvas, 
		    Tk_Item *itemPtr, 
		    double deltaX, double deltaY); 

static int
SplineIndexProc(Tcl_Interp *interp, 
		Tk_Canvas canvas, Tk_Item *itemPtr, 
		char *indexString, 
		int *indexPtr); 

static void
SplineCursorProc(Tk_Canvas canvas, 
 		    Tk_Item *itemPtr, int index); 

static int	
SplineSelectionProc(Tk_Canvas canvas, 
		       Tk_Item *itemPtr, int offset, char *buffer, 
		       int maxBytes); 
static void
SplineInsertProc(Tk_Canvas canvas, 
 		    Tk_Item *itemPtr, int beforeThis, char *string); 

static void
SplineDCharsProc(Tk_Canvas canvas, 
 		    Tk_Item *itemPtr, int first, int last); 

