/*
 * WMF drawing device for tksesh
 * This device allows one to draw on a metafile
 */

/*  2000 Serge Rosmorduc,
 * quipe EC.art, Laboratoire LRIA, Universit de Paris 8
 *
 * 15 rue Catulienne
 * 93236 St Denis Cedex
 * FRANCE
 *
 * Tout droits rservs.
 * 
 * voyez le fichier ../LICENCE.txt pour plus d'informations
 * See the ../LICENCEE.txt file for more information
 *
 * $Id: TKSO_WMF.c,v 1.1.1.1 2002/04/16 13:55:00 rosmord Exp $
 *
 */

/*
 * $Log: TKSO_WMF.c,v $
 * Revision 1.1.1.1  2002/04/16 13:55:00  rosmord
 * Imported using tkCVS
 *
 * Revision 1.3  2000/08/23 08:34:04  rosmord
 * SESHfixed suppressed.
 *
 * Revision 1.2  2000/08/04 13:43:35  rosmord
 * fixed a bug about picture size
 *
 * Revision 1.1  2000/08/04 11:49:34  rosmord
 * Initial revision
 *
 */

#define _POSIX_SOURCE 1

#include "chaine.h"
#include "tgraph.h"
#include "tgraphImpl.h"
#include "TKSO_WMF.h"
#include <tk.h>
#include "general.h"
#include <malloc.h>
#include <math.h>
#include "wmf.h"

#ifndef _GNUC_
#define inline
#endif

#ifndef PI
#define PI		3.14159265358979323846
#endif

#define TKSESHDEGREE (180*64.0)

#define WMFCAST(x) ((TKSO_WMF*)(x))

static void cleanP(TKSESH_output *);
static void DrawLineP(TKSESH_output *, TKSESH_point, TKSESH_point);
static void DrawLinesP(TKSESH_output *, int, TKSESH_point *);
static void fillPolygonP(TKSESH_output *, int , TKSESH_point * , int);
static void DrawArcP(TKSESH_output *, TKSESH_point, double, double, int, int);
static double String_widthP(TKSESH_output *, char *);
static void DrawStringP(TKSESH_output *, TKSESH_point, char *);
static void fill_rectangleP(TKSESH_output *, TKSESH_rectangle);

static TKSO_device
TKSO_WMF_driver= {
	"WMF",
	sizeof(TKSO_WMF),
	cleanP,
	DrawLineP,
	DrawLinesP,
	TKSO_genericHachureP,
	fillPolygonP,
	DrawArcP,
	String_widthP,
	DrawStringP,
	fill_rectangleP,
	0,
	0
};

/* TKSO_init --
	 Initialise a TKSESH_output. 
	 Doesn't set up a drawing surface 
	 (which is done by some TKSO_init_draw or some other function)
	 */

#define WMFBLACK RGB(0,0,0)
#define WMFRED RGB(255,0,0)
#define WMFWHITE RGB(255,255,255)

#define TBLACKPEN 0
#define TBLACKBRUSH 1
#define TREDPEN 2
#define TREDBRUSH 3
#define TWHITEPEN 4
#define TWHITEBRUSH 5
#define TWIDEBLACKPEN 6
#define TWIDEREDPEN 7
#define TSHADEBRUSH 8

static void 
fix_state_fill(TKSESH_output *o);

static void 
fix_state_draw(TKSESH_output *o);


TKSO_WMF *
TKSO_WMF_init(FILE * f,
							double x1, double y1,
							double x2, double y2, double scale)
{
	int w, h;
	TKSO_WMF *o;
	o= NOUV(TKSO_WMF);

	/* this "20" : mapping mode is in twip ? scale is supposed to be the size of a hieroglyph. */
	/* w= (x2-x1)* 20 * scale; 
		 h= (y2-y1)* 20 * scale;
	*/
	w= (x2-x1)* 20 * scale; 
	h= (y2-y1)* 20 * scale;
	
	TKSO_init_plain(TKSOC(o),x1, y1, x2, y2, w, h);

	TKSOC(o)->isLowRes= FALSE;
	TKSODRIVER(o)= &TKSO_WMF_driver;
	o->wmf= WMFinit(f, w, h, 0);
	
	WMFSetWindowOrg(o->wmf, 0, 0);
	WMFSetWindowExt(o->wmf, w, h);
	WMFSaveDC(o->wmf);
	WMFSetROP2(o->wmf, R2_COPYPEN);
	

	WMFSetBKColor(o->wmf, WMFBLACK);
	WMFSetBKMode(o->wmf, TRANSPARENT);
	
	/* Black pen */
	WMFCreatePenIndirect(o->wmf, PS_NULL, 0, WMFBLACK);
	WMFSelectObject(WMFCAST(o)->wmf, 0);
	WMFCreateBrushIndirect(o->wmf, 0, WMFBLACK, BS_SOLID);
	WMFSelectObject(WMFCAST(o)->wmf, 1);
	/* Red pen */
	WMFCreatePenIndirect(o->wmf, PS_NULL, 0, WMFRED);
	WMFSelectObject(WMFCAST(o)->wmf, 2);
	WMFCreateBrushIndirect(o->wmf, 0, WMFRED, BS_SOLID);
	WMFSelectObject(WMFCAST(o)->wmf, 3);
	/* White pen */	
	WMFCreatePenIndirect(o->wmf, PS_NULL, 0, WMFWHITE);
	WMFSelectObject(WMFCAST(o)->wmf, 4);
	WMFCreateBrushIndirect(o->wmf, 0, WMFWHITE,  BS_SOLID);
	WMFSelectObject(WMFCAST(o)->wmf, 5);
	/* wide black pen */
	WMFCreatePenIndirect(o->wmf, PS_SOLID, 12, WMFBLACK);
	WMFSelectObject(WMFCAST(o)->wmf, 6);
	/* wide red pen */
	WMFCreatePenIndirect(o->wmf, PS_SOLID, 12, WMFRED);
	WMFSelectObject(WMFCAST(o)->wmf, 7);
	/* brush for shading*/
	WMFCreateBrushIndirect(o->wmf, HS_BDIAGONAL, WMFBLACK,  BS_HATCHED);
	WMFSelectObject(WMFCAST(o)->wmf, 8);
	return o;
}

void
TKSO_WMF_free(TKSO_WMF *o)
{
	WMFDeleteObject(o->wmf, 0);
	WMFDeleteObject(o->wmf, 1);
	WMFDeleteObject(o->wmf, 2);
	WMFDeleteObject(o->wmf, 3);
	WMFDeleteObject(o->wmf, 4);
	WMFDeleteObject(o->wmf, 5);
	WMFDeleteObject(o->wmf, 6);
	WMFDeleteObject(o->wmf, 7);
	WMFDeleteObject(o->wmf, 8);
	WMFRestoreDC(o->wmf, -1);
	WMFFinish(o->wmf);
}

WMFpoint
TKSESH_point_to_WMFpoint(TKSO_WMF *o, TKSESH_point p)
{
	WMFpoint m;
	
	/* TKSESH_transformation *tr= o->trans;
		 for (i= 0; i< o->nbtrans; i++)
		 p= TKSESH_apply_transformation(&p, tr[i], p);
		 */
	m.x= (int)((p.x - TKSOWIN(o).bottom_left.x)* TKSOSCALE(o));
	m.y= (int)((TKSOWIN(o).top_right.y - p.y)* TKSOSCALE(o));
	return m;
}

static void 
cleanP(TKSESH_output * o)
{
	/* Do nothing */
}

static void 
DrawLineP(TKSESH_output * o, TKSESH_point a, TKSESH_point b)
{
	WMFpoint a1, b1;
	a1= TKSESH_point_to_WMFpoint(WMFCAST(o), a);
	b1= TKSESH_point_to_WMFpoint(WMFCAST(o), b);
	WMFMoveTo(WMFCAST(o)->wmf, a1.x, a1.y);
	fix_state_draw(o);
	WMFLineTo(WMFCAST(o)->wmf, b1.x, b1.y);
}

static void 
DrawLinesP(TKSESH_output * o, int np, TKSESH_point * l)
{
	WMFpoint m;
	int i;
	static Chaine chp= NULL;

	fix_state_draw(o);
	if (chp == empty_chaine) 
		init_Chaine(&chp, sizeof(WMFpoint));
	
	for (i=0; i< np; i++)
		{
			m= TKSESH_point_to_WMFpoint(WMFCAST(o), l[i]);
			add_to_Chaine(chp, &m);
		}
	
	WMFPolyLine(WMFCAST(o)->wmf, Chaine_to_data(chp), np);
	reset_Chaine(chp);
}

static void 
fillPolygonP(TKSESH_output * o, int nl, TKSESH_point * l, int sens)
{
	WMFpoint m;
	int i;
	TKSO_state s= o->state;
	static Chaine chp= NULL;

	if (chp == empty_chaine) 
		init_Chaine(&chp, sizeof(WMFpoint));
	
	if (!sens)
		TKSO_set_state(o, white);
	fix_state_fill(o);
 
	for (i=0; i< nl; i++)
		{
			m= TKSESH_point_to_WMFpoint(WMFCAST(o), l[i]);
			add_to_Chaine(chp, &m);
		}
	
	WMFPolygon(WMFCAST(o)->wmf, Chaine_to_data(chp), nl);
	reset_Chaine(chp);
	TKSO_set_state(o, s);
}

/*
 * See tgraph.h for more info on DrawArcP
 */

static void 
DrawArcP(TKSESH_output * o, TKSESH_point topLeft, double w, double h, int a1, int a2)
{
	/*
	 * Well, this one is a bit tricky.
	 */
	TKSESH_point center, bottomRight, start, end;
	WMFpoint topleft1, bottomright1, start1, end1;
	
	/* 
	 * We start getting rid of SESHfixed. 386 computers without
	 * FPU are getting uncommon
	 * NOW, SESHfixed == double
	 */
	center.x= topLeft.x + w/2;
	center.y= topLeft.y - h/2;
	
	bottomRight.x = center.x + w / 2;
	bottomRight.y= center.y - h / 2;
	/* xstart, ystart define a line from c with angle a1.
	 * For example, 
	 * xstart= cos(a1) * w + center.x
	 * ystart= sin(a1) * w + center.y (note that the w is NOT an error)
	 * of course, then, a1 is expressed as degrees * 64.
	 */
	start.x= cos((a1/TKSESHDEGREE)*PI) * w/2 + center.x;
	start.y= sin((a1/TKSESHDEGREE)*PI) * h/2 + center.y;
	end.x=  cos(((a1+a2)/TKSESHDEGREE)*PI) * w/2 + center.x;
	end.y= sin(((a1+a2)/TKSESHDEGREE)*PI) * h/2 + center.y;
	/* Now, all we have to to is to translate these points */
	topleft1= TKSESH_point_to_WMFpoint(WMFCAST(o), topLeft);
	bottomright1= TKSESH_point_to_WMFpoint(WMFCAST(o), bottomRight);
	start1= TKSESH_point_to_WMFpoint(WMFCAST(o), start);
	end1= TKSESH_point_to_WMFpoint(WMFCAST(o), end);
	/* We draw the arc */

	fix_state_draw(o);
	WMFArc(WMFCAST(o)->wmf,
				 topleft1.x, topleft1.y,
				 bottomright1.x, bottomright1.y,
				 start1.x, start1.y,
				 end1.x, end1.y);
}

static double 
String_widthP(TKSESH_output * o, char * txt)
{
	/* 
	 * Rather random try.
	 */
	return 35.0*strlen(txt)*2/TKSOSCALE(o);
}

static void 
DrawStringP(TKSESH_output * o, TKSESH_point p, char * txt)
{
	WMFpoint pos;
	p.y= p.y + 0.5;
	pos= TKSESH_point_to_WMFpoint(WMFCAST(o), p);
	WMFTextOut(WMFCAST(o)->wmf, pos.x, pos.y, txt);
}

static void 
fill_rectangleP(TKSESH_output * o, TKSESH_rectangle r)
{
	WMFpoint bottomleft, topright;
	return;
	bottomleft= TKSESH_point_to_WMFpoint(WMFCAST(o), r.bottom_left);
	topright= TKSESH_point_to_WMFpoint(WMFCAST(o), r.top_right);
	WMFRectangle(WMFCAST(o)->wmf, bottomleft.x, topright.y, topright.x, bottomleft.y);
}

static void 
fix_state_draw(TKSESH_output *o)
{
	switch(o->state)
		{
		case white :
			WMFSelectObject(WMFCAST(o)->wmf, TWHITEPEN);
			break;
		case red :
			WMFSelectObject(WMFCAST(o)->wmf, TWIDEREDPEN);
			break;
		case gram :
		case gramend :
		case blue :
		case aux :
		case grayfill :
		case invert :
		case black:
			WMFSelectObject(WMFCAST(o)->wmf, TWIDEBLACKPEN);
			break;
		}
}

static void 
fix_state_fill(TKSESH_output *o)
{
	switch(o->state)
		{
		case white :
			WMFSelectObject(WMFCAST(o)->wmf, TWHITEBRUSH);
			break;
		case red :
			WMFSelectObject(WMFCAST(o)->wmf, TREDPEN);
			WMFSelectObject(WMFCAST(o)->wmf, TREDBRUSH);
			break;
		case gram :
		case gramend :
		case blue :
		case aux :
		case grayfill :
		case invert :
		case black:
			WMFSelectObject(WMFCAST(o)->wmf, TBLACKPEN);
			WMFSelectObject(WMFCAST(o)->wmf, TBLACKBRUSH);
			break;
		}
}

/* Let's preserve the present presentation
 * Local Variables:
 * tab-width: 2
 * c-block-comments-indent-p: nil
 * End:
 */
