/*
 * Tkwin drawing device for tksesh
 * This device allows one to draw on a tk widget.
 */

/*  1998, 2000 Serge Rosmorduc,
 * quipe EC.art, Laboratoire LRIA, Universit de Paris 8
 *
 * 15 rue Catulienne
 * 93236 St Denis Cedex
 * FRANCE
 *
 * Tout droits rservs.
 * 
 * voyez le fichier ../LICENCE.txt pour plus d'informations
 * See the ../LICENCEE.txt file for more information
 *
 * $Id: TKSO_tkwin.c,v 1.1.1.1 2002/04/16 13:55:00 rosmord Exp $
 *
 */

/*
 * $Log: TKSO_tkwin.c,v $
 * Revision 1.1.1.1  2002/04/16 13:55:00  rosmord
 * Imported using tkCVS
 *
 * Revision 1.2  2000/08/23 08:34:05  rosmord
 * SESHfixed suppressed.
 *
 * Revision 1.1  2000/08/04 11:50:37  rosmord
 * Initial revision
 *
 */

#define _POSIX_SOURCE 1

#include "chaine.h"
#include "TKSO_tkwin.h"
#include "tgraphImpl.h"
#include "general.h"
#include <malloc.h>

#ifndef _GNUC_
#define inline
#endif

#define TKWC(x) ((TKSO_tkwin*)(x))
static char * nom_fonte= "Times 14";

static unsigned int abime_width= 16;
static unsigned int abime_height= 16;

static unsigned char abime_bits[] = {
   0x88, 0x88, 0x44, 0x44, 0x22, 0x22, 0x11, 0x11, 0x88, 0x88, 0x44, 0x44,
   0x22, 0x22, 0x11, 0x11, 0x88, 0x88, 0x44, 0x44, 0x22, 0x22, 0x11, 0x11,
   0x88, 0x88, 0x44, 0x44, 0x22, 0x22, 0x11, 0x11};

static void cleanP(TKSESH_output *);
static void DrawLineP(TKSESH_output *, TKSESH_point, TKSESH_point);
static void DrawLinesP(TKSESH_output *, int, TKSESH_point *);
static void hachureP(TKSESH_output *, TKSESH_rectangle rect);
static void fillPolygonP(TKSESH_output *, int , TKSESH_point * , int);
static void DrawArcP(TKSESH_output *, TKSESH_point, double, double, int, int);
static double String_widthP(TKSESH_output *, char *);
static void DrawStringP(TKSESH_output *, TKSESH_point, char *);
static void fill_rectangleP(TKSESH_output *, TKSESH_rectangle);

static TKSO_device
TKSO_tkwin_driver= {
	"tkwin",
	sizeof(TKSO_tkwin),
	cleanP,
	DrawLineP,
	DrawLinesP,
	hachureP,
	fillPolygonP,
	DrawArcP,
	String_widthP,
	DrawStringP,
	fill_rectangleP,
	0,
	0
};

/* 
	 Potential memory leaks :
	 
	 A) data part.
	 
	 B) palette part : No.
	 Palettes are created and deleted by the sesh widget itself.

*/

//#ifndef NEW
//#define NEW(x) (x *)ALLOUER(sizeof(x))
//#endif

#ifndef __GNUC__
#define inline
#endif

static void
prepare_les_gcs(TKSO_tkwin *o,
								TKSESH_palette *p);
static void 
TKSO_validate_state(TKSO_tkwin *o);


/* TKSO_init --
	 Initialise a TKSESH_output. 
	 Doesn't set up a drawing surface 
	 (which is done by some TKSO_init_draw or some other function)
	 */

TKSO_tkwin *
TKSO_tkwin_init(Display *d,									 
								double x1, double y1,
								double x2, double y2,
								int w, int h,
								Tk_Window win,
								Tcl_Interp *interp)
{
	TKSO_tkwin *o;
	o= NOUV(TKSO_tkwin);
	TKSO_init_plain((TKSESH_output *)o,x1,y1,x2,y2,w,h);

	TKSODRIVER(o)= &TKSO_tkwin_driver;
	o->display= d;
	o->d= None;
	o->p= NULL;
	o->gctouse= None;
	o->win= win;
	o->interp= interp;
	o->is_attached= FALSE;
	TKSOC(o)->isLowRes= TRUE;						/* we could compute this against the scale ! */
	return o;
}



void
TKSO_tkwin_free(TKSO_tkwin *o)
{
	LIBERE(o);
}



/*
	Attach a TKSO_tkwin to the 
	widget's displaying surface.
*/

void
TKSO_tkwin_init_draw(TKSO_tkwin *o)
{
	o->is_attached= TRUE;
	o->d= Tk_WindowId(o->win);
	if (!o->p->complete)
		{
			prepare_les_gcs(o, o->p);
			o->p->complete= TRUE;
		}
}



/*
	TKSO_free_draw --
	Frees a TKSO_tkwin
	*/

void
TKSO_tkwin_free_draw(TKSO_tkwin *o)
{
	/* 1) Free  old Pixmap */
	/* (Only if there is something to free
		 and if the pixmap won't be freed by other routines) */
	if (o->d != None && ! o->is_attached)
		{
			Tk_FreePixmap(o->display, o->d);
			o->d= None;
		}
}


void
TKSO_tkwin_copy(TKSO_tkwin *a, TKSO_tkwin *b)
{
	XCopyArea(a->display, 
						a->d, 
						b->d,
						b->p->copygc,
						0, 0, TKSOW(a), TKSOH(a),
						0, 0); 
}



static void
cleanP(TKSESH_output *o)
{
	TKSO_state old;
	old= TKSO_get_state(o);
	TKSO_set_state(o, white);
	TKSO_validate_state(TKWC(o));
	XFillRectangle(TKWC(o)->display, TKWC(o)->d, TKWC(o)->gctouse, 0, 0,
								 o->w,
								 o->h);
	TKSO_set_state(o, old);	
}


static void
fill_rectangleP(TKSESH_output *o, TKSESH_rectangle r)
{
	int w, h;
	XPoint aa, bb;
	aa= TKSESH_point_to_XPoint(o, r.bottom_left);
	bb= TKSESH_point_to_XPoint(o, r.top_right);
	w= (aa.x - bb.x);
	w = (w>0)?w:(-w);
	h= (aa.y - bb.y);
	h = (h>0)?h:(-h);
	
	TKSO_validate_state(TKWC(o));
	
	XFillRectangle(TKWC(o)->display,
								 TKWC(o)->d,
								 TKWC(o)->gctouse, 
								 aa.x, 
								 bb.y,
								 w,
								 h);
}


static void
DrawLineP(TKSESH_output *destination,
					TKSESH_point a,
					TKSESH_point b)
{
	XPoint aa, bb;

	aa= TKSESH_point_to_XPoint(destination, a);
	bb= TKSESH_point_to_XPoint(destination, b);
	
	TKSO_validate_state(TKWC(destination));
	XDrawLine(TKWC(destination)->display, 
						TKWC(destination)->d,
						TKWC(destination)->gctouse,
						aa.x, aa.y, bb.x, bb.y);
}


static void
hachureP(TKSESH_output *destination,
				 TKSESH_rectangle r)
{
#ifndef MAC_TCL
	int w, h;
	XPoint aa, bb;
	aa= TKSESH_point_to_XPoint(destination, r.bottom_left);
	bb= TKSESH_point_to_XPoint(destination, r.top_right);
	w= (aa.x - bb.x);
	w = (w>0)?w:(-w);
	h= (aa.y - bb.y);
	h = (h>0)?h:(-h);

	TKSO_validate_state(TKWC(destination));
	XFillRectangle(TKWC(destination)->display, 
								 TKWC(destination)->d,
								 TKWC(destination)->p->graygc, 
								 aa.x, 
								 bb.y,
								 w,
								 h);
#else
	TKSO_genericHachureP(destination, r);
#endif /* ifdef MAC_TCL */
}


static void
fillPolygonP(TKSESH_output *destination,
						 int npoints, 
						 TKSESH_point * lespoints,
						 int sens)
{
	XPoint m;
	int i;
	static Chaine chp= NULL;

	if (chp == empty_chaine) 
		init_Chaine(&chp, sizeof(XPoint));
	
	for (i=0; i< npoints; i++)
	{
		m= TKSESH_point_to_XPoint(destination, lespoints[i]);
		add_to_Chaine(chp, &m);
	}
	
	if (sens)
		{
			TKSO_validate_state(TKWC(destination));
			XFillPolygon(TKWC(destination)->display, 
									 TKWC(destination)->d,
									 TKWC(destination)->gctouse, 
									 Chaine_to_data(chp), npoints, Nonconvex, CoordModeOrigin);
		}
	else
		{
			TKSO_state old;
			old= TKSO_get_state(destination);
			TKSO_set_state(destination, white);
			TKSO_validate_state(TKWC(destination));
			XFillPolygon(TKWC(destination)->display, 
									 TKWC(destination)->d,
									 TKWC(destination)->gctouse, 
									 Chaine_to_data(chp), npoints, Nonconvex, CoordModeOrigin);
			TKSO_set_state(destination, old);	
		}
	reset_Chaine(chp);
}



static void
DrawArcP(TKSESH_output *destination,
				 TKSESH_point center,
				 double width,
				 double height,
				 int angle1, int angle2)
{
	XPoint c;
	int w, h;
	double sc;
	sc= destination->scale;
	c= TKSESH_point_to_XPoint(destination, center);
	w= sc*width;
	h= sc*height;
	TKSO_validate_state(TKWC(destination));
	XDrawArc(TKWC(destination)->display, 
					 TKWC(destination)->d, 
					 TKWC(destination)->gctouse, 
					 c.x, 
					 c.y,
					 w,
					 h,
					 angle1, 
					 angle2);
}


static void
DrawLinesP(TKSESH_output *destination,
					 int npoints, 
					 TKSESH_point * lespoints)
{
	XPoint m;
	int i;
	static Chaine chp= NULL;

	TKSO_validate_state(TKWC(destination));

	if (chp == empty_chaine) 
		init_Chaine(&chp, sizeof(XPoint));
	
	for (i=0; i< npoints; i++)
		{
			m= TKSESH_point_to_XPoint(destination, lespoints[i]);
			add_to_Chaine(chp, &m);
		}
	
	XDrawLines(TKWC(destination)->display, 
						 TKWC(destination)->d,
						 TKWC(destination)->gctouse,
						 Chaine_to_data(chp), npoints, CoordModeOrigin);
	reset_Chaine(chp);
}


static void
DrawStringP(TKSESH_output *destination,
						TKSESH_point ou,
						char *texte)
{
	XPoint m;
	m= TKSESH_point_to_XPoint(destination, ou);

	TKSO_validate_state(TKWC(destination));
	Tk_DrawChars(TKWC(destination)->display, 
							 TKWC(destination)->d,
							 TKWC(destination)->p->textgc,
							 TKWC(destination)->p->fonttouse,
							 texte, strlen(texte),
							 m.x, m.y);
}



static double
String_widthP(TKSESH_output * destination,
							char *texte)
{
	int d;
	if (TKWC(destination)->p->complete)
		d= Tk_TextWidth(TKWC(destination)->p->fonttouse, texte, strlen(texte));
	else 
		d= 0;
	return d/destination->scale;
}

static inline int TKSESH_abs(int x)
{
	return (x<0) ? -x : x;
}


void
TKSO_tkwin_trans_copie_fenetre(TKSO_tkwin *depart,
															 TKSO_tkwin *arrivee,
															 TKSESH_rectangle fenetre,
															 TKSESH_point destination)
{
	XPoint bas, haut, dest;
	int wd, ht;
	
	bas= TKSESH_point_to_XPoint(TKSOC(depart), fenetre.bottom_left);
	haut= TKSESH_point_to_XPoint(TKSOC(depart), fenetre.top_right);
	dest= TKSESH_point_to_XPoint(TKSOC(arrivee), destination);
	wd= TKSESH_abs(haut.x-bas.x);
	ht= TKSESH_abs(haut.y-bas.y);
	dest.y -= ht;
	
	XCopyArea(depart->display, depart->d, 
						arrivee->d, arrivee->p->copygc, 
						bas.x, haut.y,wd, ht,
						dest.x, dest.y);
}
						


static void
prepare_les_gcs(TKSO_tkwin *o,
								TKSESH_palette *p)
{
	XGCValues gcval;
	int i;
	/* first, color GC : */
	for (i= 0; i < p->ncolors; i++)
		{
			gcval.function= GXcopy;
			gcval.foreground = p->colors[i]->pixel;
			gcval.background = p->colors[i]->pixel;
			p->gcs[i]= Tk_GetGC(o->win, 
													GCFunction|GCForeground|GCBackground,
													&gcval);
		}
	
	gcval.function= GXcopy;
	p->copygc= Tk_GetGC(o->win, 
											GCFunction,
											&gcval);
		
	gcval.fill_style= FillStippled;
	gcval.stipple= XCreateBitmapFromData(o->display,
																			 o->d,
																			 (char *)abime_bits,
																			 abime_width,
																			 abime_height
																			 );
	gcval.function= GXcopy;
	gcval.foreground = p->colors[1]->pixel;
	p->graygc= Tk_GetGC(o->win, 
											GCFunction|GCStipple|GCFillStyle,
											&gcval);
		
	/* gcval.function= GXxor; doesn't work with MSwindows*/
	/*gcval.function= GXorReverse;*/
	gcval.foreground= p->colors[white]->pixel;
	gcval.background= p->colors[black]->pixel;
	p->invertgc= Tk_GetGC(o->win,
												GCForeground|GCBackground,
												&gcval);

	/* To be changed in a while for multiple fonts handling and the like */
	
	p->fonttouse= Tk_GetFont(o->interp, o->win, nom_fonte);

	gcval.font= Tk_FontId(p->fonttouse);
	p->textgc= Tk_GetGC(o->win, 
											GCFont, &gcval);
}


PUBLIC TKSESH_palette *
TKSO_tkwin_new_palette(int colc,
									 XColor **colv)
{
	TKSESH_palette *res;
	int i;
	res= NOUV(TKSESH_palette);
	res->complete= FALSE;
	res->ncolors= colc;
	res->colors= (XColor **)ALLOUER(sizeof(XColor*)*colc);
	for (i= 0; i< colc; i++)
		res->colors[i]= colv[i];
	/* colored GC */
	res->gcs= (GC *)ALLOUER(sizeof(GC)*colc);
	return res;
}


void
TKSO_tkwin_free_palette(Display *d, TKSESH_palette *p)
{
	if (p) {
		int i;
		if (p->complete) 
			{
				for (i= 0; i < p->ncolors; i++)
					Tk_FreeGC(d, p->gcs[i]);
				/*Tk_FreeGC(o->display,p->erasegc);*/
				Tk_FreeGC(d,p->copygc);
				Tk_FreeGC(d,p->graygc);
				Tk_FreeGC(d,p->invertgc);
				Tk_FreeGC(d,p->textgc);

				Tk_FreeFont(p->fonttouse); 
			}
		LIBERE(p->colors);
		LIBERE(p);
	}
}

/*void
	TKSO_free_palette(TKSO_tkwin *o)
	{
	if (o->p)
	TKSESH_free_palette(o, o->p);
	}
*/

PUBLIC void
TKSO_tkwin_set_palette(TKSO_tkwin *o, TKSESH_palette * palette)
{
	if (o->p != NULL)
		{
			fprintf(stderr, "erreur. Palette dj cre ????\n");
			exit(1);
		}
	// TKSESH_free_palette(o, o->p); // NOW, THIS IS A BUG IF TWO WIDGETS
																//INTEND TO USE DIFFERENT PALETTES
	                              // WE SHOULD USE REFERENCE COUNTING
	o->p= palette;
}

/* 
	 This function actually chooses
	 the graphic context to use according 
	 to the current conditions
*/

static void 
TKSO_validate_state(TKSO_tkwin *o)
{
	TKSO_state s;
	s= TKSOSTATE(o);
	switch(s)
		{
		case white :
			if (TKSOREVERSE(o)) 
				o->gctouse= o->p->gcs[black];
			else
				o->gctouse= o->p->gcs[white];
			break;
		case black:
			if (TKSOREVERSE(o)) 
				o->gctouse= o->p->gcs[white];
			else
				o->gctouse= o->p->gcs[black];
			break;
		case gram :
		case gramend :
		case red :
		case blue :
		case aux :
		case ignore:
			o->gctouse= o->p->gcs[s];
			break;
		case grayfill :
			o->gctouse= o->p->graygc;
			break;
		case invert :
			o->gctouse= o->p->invertgc;
			break;
		}
}

/*
 * Memory only copy of TKSO_tkwin.
 */

void
TKSO_tkwin_plain_copy(TKSO_tkwin *o, TKSO_tkwin *orig)
{
	*o= *orig;		/* copie des valeurs */
}

void
TKSO_tkwin_steal_copy(TKSO_tkwin *o, 
											TKSO_tkwin *orig)
{
	TKSO_tkwin_plain_copy(o,orig);
	orig->d= None;
	orig->is_attached= FALSE;	
}

/* 
	 we need a copy of orig in new for double buffering purposes.  Old
	 is some old TKSESH_output we don't use anymore, but which may hold just
	 the good resources for our need.

	 If old is not suitable, a new output is created and old's ressources are freed.
*/

int
TKSO_tkwin_recycle(TKSO_tkwin *newout, TKSO_tkwin *orig, TKSO_tkwin *old)
{
	int res;
	if (TKSOW(old) == TKSOW(orig) && TKSOH(old) == TKSOH(orig))
		{
			TKSO_tkwin_plain_copy(newout,orig);
			/* except is_attached !! */
			newout->is_attached= FALSE;
			/* we steal old's drawable */
			newout->d= old->d;
			/* so poor old has no drawable */
			old->d= None;
			/* we erase the new screen */
			TKSO_clean(TKSOC(newout));	
			res= TRUE;
		}
	else
		{
			res= TKSO_tkwin_get_working_copy(newout, orig);
		}
	/* If old has still some graphical resources, free them */
	TKSO_tkwin_free_draw(old);
	return res;
}


int
TKSO_tkwin_get_working_copy(TKSO_tkwin *o, TKSO_tkwin *orig)
{
	/* We copy the whole data */
	TKSO_tkwin_plain_copy(o,orig);
	/* except is_attached !! */
	o->is_attached= FALSE;

	o->d= Tk_GetPixmap(orig->display, 
										 orig->d,
										 TKSOW(orig), TKSOH(orig),
										 Tk_Depth(orig->win));
	

	if (!o->d)
		return FALSE;
	
	/* if (SESH_blackgc ==  None)
		 prepare_les_gcs(o,o->d);  init gc */
	
	TKSO_clean(TKSOC(o));	/* effacage */
	return TRUE;
}


int
TKSO_tkwin_is_ready(TKSO_tkwin *o)
{
	return (o->d != None);
}

XImage *
TKSO_tkwin_getImage(TKSO_tkwin * o)
{
	XImage *imagePtr;
	imagePtr= XGetImage(Tk_Display(o->win),
											o->d, 0, 0,
											TKSOW(o), TKSOH(o),
											AllPlanes, XYPixmap);
	return imagePtr;
}

int
TKSO_tkwin_hasPalette(TKSO_tkwin * o)
{
	return o->p != NULL;
}

/* Let's preserve the present presentation
 * Local Variables:
 * tab-width: 2
 * c-block-comments-indent-p: nil
 * End:
 */
