/* 
 *	 Font use and drawing
 $Id: bfont.c,v 1.1.1.1 2002/04/16 13:55:00 rosmord Exp $	 
 */


#define _POSIX_SOURCE 1

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>							/* lots of functions */
#include <strings.h>

#include "general.h"
#include "duplique.h"
#include "bzrop.h"
#include "chaine.h"
#include "bfont.h"

#ifndef NOUV
#define NOUV(x) (x *)ALLOUER(sizeof(x))
#endif
#define MAXREEL (255.0 + 255.0/256 + 255.0/(256*256))

/* This is a POSIX world out there.
   #ifndef SEEK_CUR
   #define SEEK_CUR 1
   #define SEEK_SET 0
   #define SEEK_END 2
   #endif
*/

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

static char lastchar;

static void 
erreur(char *s)
{
	fprintf(stderr,"%s", s);
	exit(1);
}

static double 
read_Bdouble(FILE * f)
{
	double d;
	unsigned char c[3];
	fread(c, sizeof(char), 3, f);
	d= c[0] + (double)c[1]/ (1 << 8) + (double)c[2]/ (1 << 16);
	if (c[0] & (1 << 7))
		d= d - MAXREEL;
	return d;
}

static double 
read_Sdouble(FILE * f)
{
	double d;
	unsigned char c[2];
	fread(c, sizeof(char), 2, f);
	d= (double)c[0]/ (1 << 8) + (double)c[1]/ (1 << 16);
	if (c[0] & (1 << 7))
		d= d - 1;
	return d;
}

static void 
fixe_sens_contour(Contour *cont)
{
	double val= 0.0;
	TKSESH_point* pts;
	double ox, oy, x, y, firstx, firsty;
	int i;

	pts= cont->virtuel_trace;
	firstx= ox= pts[0].x;
	firsty= oy= pts[0].y;
	for (i=1; i< cont->nbr_pts; i++)
		{
			x= pts[i].x;
			y= pts[i].y;
			val+= (x -ox) * (y + oy);
			ox=x;
			oy= y;
		}
	val+= ((firstx - ox) * (firsty + oy));
	
	if (val > 0.0)
		cont->type= negatif;
	else 
		cont->type= positif;
} 

PUBLIC void 
BZR_fixe_trace(TKSESH_point depart, 
							 TKSESH_point c1, 
							 TKSESH_point c2,
							 TKSESH_point dernier,
							 Chaine s)
{
	double ux, uy,vx, vy ;
	double wx, wy, zx, zy;
	double t, increment;
	int i;

	increment= 1.0/MAXPTS;
	
	ux= 3.0*(c1.x - c2.x)  + (dernier.x - depart.x);
	vx= 3.0*(c2.x + depart.x) - 6.0*c1.x;
	wx= 3.0*(c1.x - depart.x);
	zx= depart.x;

	uy= 3.0*(c1.y - c2.y) + (dernier.y - depart.y);
	vy= 3.0*(c2.y + depart.y) - 6.0*c1.y;
	wy= 3.0*(c1.y - depart.y);
	zy= depart.y;

	for (i= 0, t= 0.0; i<MAXPTS; i++)
		{
			TKSESH_point trace;
		
			t= t + increment;
			trace.x= (((((ux*t)+vx)*t)+wx)*t)+zx;
			trace.y= (((((uy*t)+vy)*t)+wy)*t)+zy;
			/* formule : trace.y= ((uy*t+vy)*t+wy)*t+zy;*/
			add_to_Chaine(s,&trace);
		}
}

static char
lire_un_char(FILE *f)
{
	size_t lu;
	char c;
	lu= fread(&c, sizeof(char), 1, f);
	if (lu == 0)
		return -1;
	else
		return c;
}

static char 
skip_nop(FILE *f)
{
	int c;
	while ((c=lire_un_char(f)) == NO_OP);
	lastchar= c;
	return c;
}

/* lecture en arrire pour le postamble */
/* retourne EOF en cas d'chec */
static int
backread(FILE *f)
{
	char c;
	if (fseek(f,-1L, SEEK_CUR) != EOF)
		{
			fread(&c, sizeof(char), 1, f);
			fseek(f,-1L, SEEK_CUR);
			return c;
		}
	else
		return EOF;
}

static void
BZR_read_char_dims(BFont *fonte, FILE * f)
{
	int c, pasbon, i;
	unsigned long int pointeur[256];
	int is_a_char[256];
	
	fseek(f,fonte->post,SEEK_SET);
	pasbon= FALSE;
	for (i=0; i<256; i++)
		is_a_char[i]= FALSE;

	while (! (pasbon))
		{
			unsigned char pre[2];
			fread(pre,sizeof(char),2L,f);
			switch (pre[0])
				{
					unsigned char pt[4];
				case CHAR_LOC_ABBREV:
					is_a_char[pre[1]]= TRUE;
					fread(pt,sizeof(char),2,f);
					pointeur[pre[1]]= (pt[2] << 8) + pt[3];
					break;
				case CHAR_LOC:
					is_a_char[pre[1]]= TRUE;
					fread(pt,sizeof(char),4,f);
					pointeur[pre[1]]= (pt[0] << 24) + (pt[1] << 16) +(pt[2] << 8) + pt[3];
					break;
				default:
					pasbon= TRUE;
				}
		}
	for (i=0; i<256; i++)
		{
			if (is_a_char[i])
				{
					Lettre *ltr;
					int code;

					fseek(f,pointeur[i],SEEK_SET);
	
					c=skip_nop(f);
					code= (unsigned)lire_un_char(f);
					ltr= fonte->lettres + code;
					ltr->code= code;
					ltr->defined= TRUE;

					switch(c)
						{
						case BOC_ABBREV :
							ltr->width= read_Sdouble(f) * fonte->size;
							ltr->llx= read_Sdouble(f) * fonte->size;
							ltr->lly= read_Sdouble(f) * fonte->size;
							ltr->urx= read_Sdouble(f) * fonte->size;
							ltr->ury= read_Sdouble(f) * fonte->size;
							break;
						case BOC:
							ltr->width= read_Bdouble(f) * fonte->size;
							ltr->llx=   read_Bdouble(f) * fonte->size;
							ltr->lly=   read_Bdouble(f) * fonte->size;
							ltr->urx=   read_Bdouble(f) * fonte->size;
							ltr->ury=   read_Bdouble(f) * fonte->size;
							break;
						default:
							break;
						}
				}
		}
}



static void 
BZR_read_char(BFont *fonte, unsigned char carac)
{
	/* first, go to the end of the file to find the character */
	FILE *f;
	int c, trouve, pasbon;
	unsigned long int pointeur;
	
	if ((f= fopen(fonte->filename,"rb")) == NULL)
		{
			char s[500];
			sprintf(s, "can't find font %s\n", fonte->filename);
			erreur(s);
		}
	fseek(f,fonte->post,SEEK_SET);
	trouve= FALSE;
	pasbon= FALSE;
	while (! ( trouve  || pasbon))
		{
			unsigned char pre[2];
			fread(pre,sizeof(char),2L,f);
			switch (pre[0])
				{
					unsigned char pt[4];
				case CHAR_LOC_ABBREV:
					fread(pt,sizeof(char),2,f);
					if ((trouve= (pre[1]== carac)))
						pointeur= (pt[2] << 8) + pt[3];
					break;
				case CHAR_LOC:
					fread(pt,sizeof(char),4,f);
					if((trouve= (pre[1]== carac)))
						pointeur= (pt[0] << 24) + (pt[1] << 16) +(pt[2] << 8) + pt[3];
					break;
				default:
					pasbon= TRUE;
				}
		}
	if (pasbon)
		{
			erreur("ouille ouille ouille mauvais caractre\n"); 
		}
	else
		{
			Contour *lastcont, *currcont;
			Lettre *ltr;
			int code;
		
			fseek(f,pointeur,SEEK_SET);
		
			c=skip_nop(f);
			code= (unsigned)lire_un_char(f);
			ltr= fonte->lettres + code;
			ltr->code= code;
			switch(c)
				{
				case BOC_ABBREV :
					ltr->loaded= TRUE;
					ltr->contours= NULL;
					ltr->width= read_Sdouble(f) * fonte->size;
					ltr->llx= read_Sdouble(f) * fonte->size;
					ltr->lly= read_Sdouble(f) * fonte->size;
					ltr->urx= read_Sdouble(f) * fonte->size;
					ltr->ury= read_Sdouble(f) * fonte->size;
					break;
				case BOC:
					ltr->loaded= TRUE;
					ltr->contours= NULL;
					ltr->width= read_Bdouble(f) * fonte->size;
					ltr->llx= read_Bdouble(f) * fonte->size;
					ltr->lly= read_Bdouble(f) * fonte->size;
					ltr->urx= read_Bdouble(f) * fonte->size;
					ltr->ury= read_Bdouble(f) * fonte->size;
					break;
				default:
					break;
				}
		
			/* paths */
			lastcont= NULL;
			c= skip_nop(f);
			while ( c != EOC)
				{
					static Chaine path_to_build= NULL;
					TKSESH_point lastpoint, start;

					if (!path_to_build)
						init_Chaine(&path_to_build, sizeof(TKSESH_point));
					/* assert(c== START_PATH || c == START_CURVE); */
			
					currcont= NOUV(Contour);
					currcont->next= NULL;
					switch (c) {
					case START_CURVE:
						currcont->type= trait;
						break;
					case START_NOFIL:
						currcont->type= nofill;
						break;
					default:
						currcont->type= positif;
					}

					start.x= read_Bdouble(f) * fonte->size;
					start.y= read_Bdouble(f) * fonte->size;
					lastpoint= start;	
					add_to_Chaine(path_to_build, &start);
							
					while ((c= skip_nop(f)) != START_PATH 
								 && c != START_CURVE
								 && c != START_NOFIL
								 && c != EOC)
						{
							enum {ligne, spline} type;
							TKSESH_point dernier, controle1, controle2;
				
							switch(c)
								{
								case LINE:
									type= ligne;
									dernier.x= read_Bdouble(f) * fonte->size;
									dernier.y= read_Bdouble(f) * fonte->size;
									break;
								case LINE_ABBREV:
									type= ligne;
									dernier.x= read_Sdouble(f) * fonte->size;
									dernier.y= read_Sdouble(f) * fonte->size;
									break;
								case SPLINE:
									type= spline;
									controle1.x= read_Bdouble(f) * fonte->size;
									controle1.y= read_Bdouble(f) * fonte->size;
									controle2.x= read_Bdouble(f) * fonte->size;
									controle2.y= read_Bdouble(f) * fonte->size;
									dernier.x= read_Bdouble(f) * fonte->size;
									dernier.y= read_Bdouble(f) * fonte->size;
									break;
								case SPLINE_ABBREV:
									type= spline;
									controle1.x= read_Sdouble(f) * fonte->size;
									controle1.y= read_Sdouble(f) * fonte->size;
									controle2.x= read_Sdouble(f) * fonte->size;
									controle2.y= read_Sdouble(f) * fonte->size;
									dernier.x= read_Sdouble(f) * fonte->size;
									dernier.y= read_Sdouble(f) * fonte->size;
									break;
								default:
									fprintf(stderr,"Unknown TAG : %c\n", c);
									exit(1);

								}
							switch (type)
								{
								case ligne:
									add_to_Chaine(path_to_build, &dernier);
									break;
								case spline : 
									BZR_fixe_trace(lastpoint, controle1,
																 controle2, dernier, path_to_build);
									break;
								}
							lastpoint= dernier;
						}
					currcont->nbr_pts= size_of_Chaine(path_to_build);
					currcont->virtuel_trace= (TKSESH_point *)Chaine_to_cstring(path_to_build);
					currcont->scaled_trace= (TKSESH_point *)Chaine_to_cstring(path_to_build);
			
					if (currcont->nbr_pts > 2)
						{
							if (currcont->type == positif || currcont->type == negatif) 
								fixe_sens_contour(currcont);
							if (lastcont)
								{
									lastcont->next= currcont;
									lastcont= currcont;
								}
							else
								{
									lastcont= ltr->contours= currcont;
								}
						}
					else
						{
							LIBERE(currcont);					/* pas de contour sans trait chez moi ! */
						}
					reset_Chaine(path_to_build);
				}
		}
	fclose(f);
}

BFont * 
BZR_read(char *name)
{
	BFont *fonte;
	FILE *f;
	int i, numchar;
	int c;
	double size;

	if ((f= fopen(name,"rb")) == NULL)
		return NULL;

	fonte= NOUV(BFont);
	for (i=0; i< CHRINFONT; i++)
		{
			fonte->lettres[i].loaded= FALSE;
			fonte->lettres[i].defined= FALSE;
			fonte->lettres[i].font= fonte;
		}
	fonte->filename= dupliquechaine(name);
	size= fonte->size= read_Bdouble(f);
	numchar= (unsigned)lire_un_char(f);
	/*  font name. Not really useful.
			If it is supressed, one still
			must advance in the file.
	*/
	fonte->name= 
		(char *)ALLOUER((numchar+1)*sizeof(char));
	fonte->name[numchar]=0;
	fread(fonte->name,sizeof(char),
				numchar,f);
			
			
	fseek(f, 0L, SEEK_END);				/* on va a la fin */
	while ((c=backread(f)) == NO_OP);
	if (c != EOF)
		{
			unsigned char pt[4];
			fseek(f, -5L, SEEK_END);
			fread(pt,sizeof(char),4,f);
			fonte->post= (pt[0] << 24) + (pt[1] << 16) + (pt[2] << 8) + pt[3] + 13;
		}
			
	BZR_read_char_dims(fonte, f);
	fclose(f);
	return fonte;
}

/* BFONT Printing. 
	 to ease futher development, 
	 we use a function to get 
	 a pointer to each size
*/

static TKSESH_point * 
BZR_get_contour(Lettre *l,
								Contour *ct,
								double sizex,
								double sizey,
								int angle,
								int revert,
								TKSESH_point where)
{
	int i;
	/* Simple things should go fast ! */
	if (!angle && !revert)
		for(i=0; i< ct->nbr_pts; i++)
			{
				ct->scaled_trace[i].x= 
					sizex*ct->virtuel_trace[i].x + where.x;
				ct->scaled_trace[i].y= 
					sizey*ct->virtuel_trace[i].y + where.y;
			}
	else
		{
			double width, dw;
			double height, dh;
			TKSESH_point ou;
			TKSESH_point * trace_base;
			ou= where;
			
			if (angle % 180 == 0)
				{
					dw= sizex;
					dh= sizey;
				}
			else 
				{
					dw= sizey;
					dh= sizex;
				}
			
			width= l->width*dw;
			height= (l->ury - l->lly)*dh;
			/*  symetry */
			if (revert)
				{
					double origx;
					origx= l->width;
					/* origx= width;*/
					for(i=0; i< ct->nbr_pts; i++)
						{
							ct->scaled_trace[i].x= 
								origx - ct->virtuel_trace[i].x;
							ct->scaled_trace[i].y= 
								ct->virtuel_trace[i].y;
						}				
					trace_base= ct->scaled_trace;
				}
			else
				{
					trace_base= ct->virtuel_trace;
				}
			/* then rotate */
			switch (angle)
				{
				case 90:
					ou.x= ou.x + height;
					for(i=0; i< ct->nbr_pts; i++)
						{
							double x, y;
							x= trace_base[i].x;
							y= trace_base[i].y;
							ct->scaled_trace[i].x= ou.x - sizex*y ;
							ct->scaled_trace[i].y= ou.y + sizey*x;
						}
					break;
				case 270:
					ou.y= ou.y + width;
					for(i=0; i< ct->nbr_pts; i++)
						{
							double x, y;
							x= trace_base[i].x;
							y= trace_base[i].y;						
							ct->scaled_trace[i].x= sizex*y + ou.x;
							ct->scaled_trace[i].y= ou.y - sizey*x;
						}
					break;
				case 180:
					ou.x= ou.x + width;
					ou.y= ou.y + height;
					for(i=0; i< ct->nbr_pts; i++)
						{
							double x, y;
							x= trace_base[i].x;
							y= trace_base[i].y;
							ct->scaled_trace[i].x= ou.x - sizex*x;
							ct->scaled_trace[i].y= ou.y - sizey*y;
						}
					break;
				case 0:
				case 360:
				default:
					for(i=0; i< ct->nbr_pts; i++)
						{
							double x, y;
							x= trace_base[i].x;
							y= trace_base[i].y;
							ct->scaled_trace[i].x= sizex*x + ou.x;
							ct->scaled_trace[i].y= sizey*y + ou.y;
						}
					break;
				}
		}
	return ct->scaled_trace;
}

PUBLIC void  
BZR_putc(Lettre *l,
				 double sizex,
				 double sizey,
				 int angle,
				 int revert,
				 TKSESH_output *destination,
				 TKSESH_point where)
{
	Contour *ct;
	double fixed_sizex, fixed_sizey;

	fixed_sizex= sizex;
	fixed_sizey= sizey;

	if (! l->loaded)
		BZR_read_char(l->font, l->code);
	
	if (l->loaded)
		{
			for(ct=l->contours;
					ct != NULL;
					ct=ct->next)
				{
					if (ct->type == positif || ct->type == negatif)
						TKSO_fillPolygon(destination,
														 ct->nbr_pts, BZR_get_contour(l,
																													ct,fixed_sizex,fixed_sizey,
																													angle,revert,where),
														 (ct->type == positif));
					else
						TKSO_DrawLines(destination,
													 ct->nbr_pts, BZR_get_contour(l,
																												ct,fixed_sizex, fixed_sizey,
																												angle,revert,where));						
				}
			if (TKSO_isLowRes(destination))
				{
					for(ct=l->contours;
							ct != NULL;
							ct=ct->next)
						{
							/* Should be used only for LOWRES BITMAP OUTPUT !!!*/
							if (ct->type != negatif)
								TKSO_DrawLines(destination,
															 ct->nbr_pts, BZR_get_contour(l,
																														ct,fixed_sizex, fixed_sizey,
																														angle,revert,where));
						}
				}
		}
}


double 
BZR_charWidth(Lettre *l)
{
	return l->width;
}

double 
BZR_charHeight(Lettre *l)
{
	return l->ury - l->lly ;
}

/* On-the-fly font creation. 
	 New font loading and glyph definition
*/


PUBLIC void 
BZR_delete_contour(Contour *c)
{
	LIBERE(c->virtuel_trace);
	LIBERE(c->scaled_trace);
	LIBERE(c);
}

PUBLIC void 
BZR_delete_lettre(Lettre *l)
{
	Contour *c, *next;
	l->loaded= FALSE;
	l->defined= FALSE;
	c= l->contours;
	while(c != NULL) {
		next= c->next;
		BZR_delete_contour(c);
		c= next;
	}
}

PUBLIC void 
BZR_delete(BFont *f) 
{
	int i;
	for (i=0; i< CHRINFONT; i++)
		BZR_delete_lettre(&(f->lettres[i]));
	LIBERE(f->name);
	LIBERE(f->filename);
	LIBERE(f);
}

PUBLIC BFont *
BZR_new_font() {
	BFont *f;
	int i;
	f= NOUV(BFont);
	for (i=0; i< CHRINFONT; i++)
		{
			f->lettres[i].loaded= FALSE;
			f->lettres[i].defined= FALSE;
		}
	f->filename= NULL;
	f->name= NULL;
	f->post= 0;
	return f;
}


/* Let's preserve the present presentation
 * Local Variables:
 * tab-width: 2
 * c-block-comments-indent-p: nil
 * End:
 */
