/*
 * New ligature system :
 *
 * A ligature is defined by the signs it contains, 
 * regardless of their code.
 * 
 * i.e. D&d == D&D46 == I10&D46 == ...
 *
 *
 *
 * This files deals with ligatures
 * 
 * Actually, ligatures are defined like this :
 * (in the ligature file)
 *
 * A ligature is defined by something like this :
 * LIG D&d 14 14 2  D 1 1 100 d 0.25 0.25 50 end
 *
 * the format is the following :
 * "LIG" NAME DX DY NUMBER_OF_SIGNS (SIGN X Y S)+
 * "LIG" is a reserved keyword. we may introduce new possibilities later
 *
 * NAME is the ligature identifier
 * DX and DY stands for the sign bounding box size.
 * NUMBER_OF_SIGNS stands for the number of signs in the ligature
 * each signs is introduced by :
 *   -its name
 *   -its position
 *   -its size (as a % of its normal size)
 */

#define _POSIX_SOURCE 1

#include <stddef.h>		/* NULL, size_t, and so on */
#include <stdio.h>
#include <stdlib.h>


#include "hache.h"
#include "duplique.h"
#include "quadra.h"
#include "signs.h"
#include "ligatur.h"

/**********************************************************************
 * LOW LEVEL FUNCTIONS.
 */

/*
  The definitions is done this way :
  l= ligature_create(dx,dy,nbrsigns);
	
	defines a new ligature of dim dx*dy containing nbrsigns signs
*/

PUBLIC struct Ligature *
ligature_create(int nbrsigns)
{
	struct Ligature *res= NULL;
	res= NOUV(struct Ligature);
	res->code= NULL;
	res->w=0;
	res->h=0;
	res->codesize= 0;
	res->nbr= nbrsigns;
	res->signs= (struct sign_ligpos*) 
		ALLOUER(sizeof(struct sign_ligpos)*nbrsigns);
	return res;
}

/*
	ligature_subsign_init(l, num, code, x, y, size)
	
	the num^th sign for l is code, at position x,y, size % of the
	original sign.
	
	returns TRUE if initialization was successful, else FALSE
*/

PUBLIC int
ligature_subsign_init(struct Ligature *l, int num, 
											char *code, double x, double y, double size)
{
	/* union Donnee should move and be INDEPENDANT from quadra.
		 (probably move to signs.c) yeah !*/
	union _Donnees d;
	d= seshtrouvesigne(code);
	if (signe_trouve) 
		{
			l->signs[num].x= x;
			l->signs[num].y= y;
			l->signs[num].size= size;
			l->signs[num].l= d.hieroglyphe.l; 
			return TRUE;
		}
	else
		return FALSE;
}
/*
	  ligature_make_ident(l);
*/
PUBLIC void
ligature_make_ident(struct Ligature *l)
{
	int i;
	char *s;
	/* builds an identification vector
		 from "hieroglyphic code"-independant data : l
	*/
	l->codesize= sizeof(Lettre *) * 2 * l->nbr + 1;
	l->code= (char*)ALLOUER(l->codesize);
	s= l->code;
	for(i= 0; i< l->nbr; i++)
		{
			s+= sprintf(s, "%X", (unsigned int)l->signs[i].l);
		}
	
}

/*
	  ligature_complete(l);
		makes the ligature ready to use.
		should be called for real ligatures,
		not for temporary ligatures created for searching.
*/

PUBLIC void
ligature_complete(struct Ligature *l)
{
	/* computes ligature width and height. */
	double w,h;
	int i;
	w= 0;
	h=0;
	for(i=0;i< l->nbr; i++)
		{
			double w1;
			double h1;
			w1= l->signs[i].x + 
				BZR_charWidth(l->signs[i].l)*l->signs[i].size/100.0;
			h1= l->signs[i].y + 
				BZR_charHeight(l->signs[i].l)*l->signs[i].size/100.0;
			if (h1 > h)
				h= h1;
			if (w1 > w)
				w= w1;
		}
	l->w= w;
	l->h= h;
}

PUBLIC char *
ligature_to_code(struct Ligature *l)
{
	return l->code;
}

/*
	ligature_delete(l)
*/

PUBLIC void
ligature_delete(struct Ligature *l)
{
	LIBERE(l->signs);
	LIBERE(l);
}

/* Let's preserve the present presentation
 * Local Variables:
 * tab-width: 2
 * c-block-comments-indent-p: nil
 * header: "ligatur.h"
 * 
 * End:
 */
