/****************************************************************************
	Entry system for tksesh according to the Manuel de Codage's specifications
													 Serge Rosmorduc
****************************************************************************/
%{

#define _POSIX_SOURCE 1

#include <malloc.h>
#include "general.h"
#include "quadra.h"
#include "mdctexte.h"
#include "mdccoord.h"
#include "seshaux.h"
#include "getlig.h"

char rapport_erreur_de_syntaxe[1024];
int  erreur_de_syntaxe;

static int color= 0;
static int ingray= FALSE;

extern int 
seshlex();

void 
yyerror(char *s);

#include <stdio.h>
#ifndef NULL
#define NULL 0
#endif

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

Texte *resultat;

%}

%union {
	Texte *un_texte;
	struct texte_liste {
		Texte * texte;
		Texte * last;
	} une_texte_liste;
	Quadra * un_quadra;
	Vertlist* une_vertlist;
	union _Donnees une_donnee;
	Signe un_signe;
	int un_entier;
	char * une_chaine;
	text_coords des_coordonnees;
	Typehachure une_hachure;			/* ancien systme */
	Hachures des_hachures;
	Typegroupe un_typegroupe;
	Typeseparateur un_separateur;
	struct _Signinlig *ligature;
	struct _Signe *signe;
}

%token TEXTE DEBUTHIEROGS FINHIEROGS
%token HIEROGLYPHE INCONNU
%token POINT
%token POINTPOINT  
%token KERNING  
%token DEBUTCONSTRUCTION CARTF 
%token DIEZE
%token ROMAIN 
%token GAUCHEDROITE DROITEGAUCHE
%token LEFTRIGHT RIGHTLEFT
%token INTERNALSPACE
%token HACHUREG HACHUREV HACHUREH HACHURET
%token HACHURES
%token OVERWRITE
%token SIZE ROTATION NROTATION REVERT
%token IGNORE LONGSIGN WIDECADRAT
%token TEXTESUPER POINTROUGE POINTNOIR
%token RED BLACK
%token HAPLOGRAPHIE LACUNE LIGNELACUNE
%token DSTSUPERFETATOIRE FSTSUPERFETATOIRE
%token DSTEFFACE FSTEFFACE
%token DSTDISPARU FSTDISPARU
%token DSTRAJOUTSCRIBE FSTRAJOUTSCRIBE
%token DSTRAJOUTAUTEUR FSTRAJOUTAUTEUR
%token SEPARATEUR
%token FINMOT
%token GRAMMAIRE
%token FIN_LIGNE FIN_PAGE
%token SAUT_LIGNES SAUT_PAGES
%token FIXE_LIGNE
%token LIGNE_COMMENTAIRE

%type <une_texte_liste> textes textitem
%type <un_texte>  texte
%type <un_quadra> embeddedQuadras quadras quadra sousquadra sousquadraAux 
%type <un_quadra> inhierliste uppercadrat
%type <ligature> suiteligature
%type <un_quadra> construction_parenthesee
%type <une_vertlist> sousquadras 
%type <un_entier> kerning SIZE NROTATION ROTATION REVERT
%type <un_entier> LONGSIGN
%type <un_entier> options
%type <un_entier> WIDECADRAT
%type <un_entier> grammaireopt finmotopt
%type <un_entier> SAUT_LIGNES SAUT_PAGES
%type <un_entier> texteseps textesep
%type <des_hachures> HACHURES hachureopt
%type <un_typegroupe> DEBUTCONSTRUCTION
%type <une_donnee> HIEROGLYPHE INCONNU POINTNOIR POINTROUGE
%type <une_donnee> TEXTESUPER ROMAIN
%type <un_separateur> SEPARATEUR separateur separation startinggarbage
%type <des_coordonnees> FIXE_LIGNE
%type <une_chaine> LIGNE_COMMENTAIRE
%type <signe> hieroglyphes hieroglyphe lonesign signe
%type <signe> modificateurs modificateur
%left ':'  
%left '*'
%left OVERWRITE
%left '&'  

%%

hierogs					: startinggarbage textes separation {
	resultat = $2.texte;
}
                | separation {resultat= NULL;}
                | error {resultat= NULL;}
;


startinggarbage : separation
|FINMOT separation {$$=$2;}
;

textes        :   textitem
{
	$$.texte= $1.texte;
	$$.last=  $1.last;
	/* kerning ignor */
}
| textes separation textitem
{
	$$.texte=$1.texte;
	$$.last->next= $3.texte;
	$3.texte->previous= $$.last;
	$$.last=$3.last;
}
;

textitem : kerning texte 
{
	$$.texte= $2;
	$$.last= $2;
	if (ingray)
		SET_GRIS($2);
	if (color == 1)
		$2->couleur= red;
}
;

separation:
separateur texteseps;

texteseps:
/* empty */ {$$=0;}
|texteseps textesep
;

textesep:
DIEZE {
	ingray= 1-ingray;
	$$= 0;
}
|RED {
	color= 1;
	$$=0;
}
|BLACK {
	color= 0;
	$$=0;
}
;


texte         : quadra hachureopt
{
	$$= NewTexte();
	$$->q= $1;
	SESH_creer_listesignes($1, &($$->lessignes));
	$$->q->hachures= $2;
}
                | FIN_PAGE
{
	$$= NewTexte();	
	$$->type= T_NP;
	$$->saut= 0;
}
                | FIN_LIGNE
{
	$$= NewTexte();	
	$$->type= T_NL;
	$$->saut= 0;
}
| FIXE_LIGNE
{
	$$= NewTexte();	
	$$->type= T_FIXE_COORDONNEES;
	$$->coordonnees= $1;
}
| LIGNE_COMMENTAIRE
{
	$$= NewTexte();
	$$->type= T_LIGNE_COMMENTAIRE;
	$$->texte= $1; 
}
;

hachureopt :
HACHURES {$$= $1;}
| /*empty*/ {$$= NOHACHURE;}
;

uppercadrat : separation kerning quadra hachureopt
{
	$$= $3;
	$$->hachures= $4;
	if (ingray)
		$$->est_grise= TRUE;
	$$->kerning= $2;								/* nombre de signes de kerning */
	$$->separateur= $1;	
}
;

embeddedQuadras: quadras separation
{
	$$= $1;
}
;

quadras					: uppercadrat
{
	$$= $1;
}
								| quadras uppercadrat
{
	$$=$1;
	$$->last->next= $2;
	$2->previous= $$->last;
	$$->last=$2;
}
;

separateur: 
/* empty */ { $$= S_intersignes; }
|SEPARATEUR 
;

quadra					: sousquadras
{
	if (S_NEXT_OF(*($1)) == NULL
			&& S_NEXT_OF(*VL_QUADRA_OF(*($1))) == NULL
			&& (S_TYPE_OF(*VL_QUADRA_OF(*($1))) != S_quadra
					&&
					S_TYPE_OF(*VL_QUADRA_OF(*($1))) != S_signe))
	{															
		/* suppression des informations superflues */
		$$= VL_QUADRA_OF(*($1));
		VL_QUADRA_OF(*($1))= NULL;
		detruit_vertlist($1);
 	}
	else
	{
		$$= new_quadra(S_quadra);
		$$->kerning= 0;
		$$->contenu.horizontal= $1;
	}
}
                 | lonesign
{
	$$= new_quadra(S_signe);
	$$->contenu.un_signe= $1;
}
;

lonesign:          ROMAIN
{
	$$= new_sign(S_romain);
	$$->donnees= $1;
}
								 | HAPLOGRAPHIE
{
	$$= new_sign(S_haplographie);
}
								 | GAUCHEDROITE
{
	$$= new_sign(S_gauchedroite);
}
								 | DROITEGAUCHE
{
	$$= new_sign(S_droitegauche);
}
								 | LACUNE
{
	$$= new_sign(S_lacune);
}
								 | LIGNELACUNE
 {
	 $$= new_sign(S_lignelacune);
 }
								 | TEXTESUPER
 {
	 $$= new_sign(S_textesuper);
	 $$->donnees= $1;
 }
 ;

sousquadras			: sousquadra
 {
	 $$= new_vert();
	 $$->un_quadra= $1;
 }
								 | sousquadras  ':' options kerning sousquadra
 {
	 $$=$1;
	 $$->last->next= new_vert();
	 $$->last->next->previous= $$->last;
	 $$->last= $$->last->next;
	 $$->last->un_quadra= $5;
	 $$->last->kerning= $4;
	 /* False, but just to try */
	 $$->last->kerning= $3;
 }
 ;

/* Square brackets indicate options 
 * Shouldn't return an integer, but we are just trying.
 * Result : same as [size=inf]
 */
options : /* empty */ {$$=0;}
| '[' ']' {$$=1;} 
;

sousquadra : sousquadraAux {$$= $1;};

sousquadraAux			: sousquadraAux '*' options kerning inhierliste
 {
	 $$= $1;
	 $$->last->next= $5;
	 $$->last->separateur= S_intersignes;
	 $$->last->next->previous= $$->last;
	 $$->last= $5;
	 $$->last->kerning= $4;
 }
								 | inhierliste
 {
	 $$= $1;
 }
 ;

inhierliste			: 
                 hieroglyphes
{
	$$= new_quadra(S_signe);
	$$->contenu.un_signe= $1;
}
                |hieroglyphes '&' suiteligature
{
	$$= new_quadra(S_ligature);
	$$->contenu.lig= new_liggroup();
	$$->contenu.lig->l= NULL;
	$$->contenu.lig->nbr= 1 + $3->nbr;
	$$->contenu.lig->signs= new_signinlig();
	$$->contenu.lig->signs->nbr= 1;
	$$->contenu.lig->signs->sign= $1;
	$$->contenu.lig->signs->next= $3;
	fix_ligature($$->contenu.lig);
}
								| DEBUTCONSTRUCTION embeddedQuadras CARTF options
{	
	$$= new_quadra($1);
	$$->contenu.des_quadras= $2;
	$$->kerning= 0;	
}
								| RIGHTLEFT embeddedQuadras LEFTRIGHT
{	
	$$= new_quadra(S_rightleft);
	$$->contenu.des_quadras= $2;
	$$->kerning= 0;	
}
								| construction_parenthesee
								| '(' embeddedQuadras ')'
{
	$$= $2;
}
;


kerning: /* empty */
{
	$$ =0;
}
                | kerning KERNING
{
	$$= $1 + 1;
}
;

hieroglyphes    : hieroglyphe
| hieroglyphe OVERWRITE hieroglyphe
{
	$$= new_sign(S_overwrite);
	$$->donnees.over[0]= $1;
	$$->donnees.over[1]= $3;
}
;

suiteligature:
hieroglyphes
{
	$$= new_signinlig();
	$$->nbr= 1;
	$$->sign= $1;
	$$->next= NULL;
}
| hieroglyphes '&' suiteligature
{
	$$= new_signinlig();
	$$->nbr= 1 + $3->nbr;
	$$->sign= $1;
	$$->next= $3;	
}
;	


signe  			: HIEROGLYPHE
{
	$$= new_sign(S_hieroglyphe);
	$$->donnees= $1;
}
| POINT
 {
	 $$= new_sign(S_point);
 }
| POINTPOINT
{
	$$= new_sign(S_pointpoint);
}
| POINTROUGE
{
	$$= new_sign(S_pointrouge);
	$$->donnees= $1;
}
| POINTNOIR
{
	$$= new_sign(S_pointnoir);
	$$->donnees= $1;
}
| INCONNU
{
	$$= new_sign(S_inconnu);
	$$->donnees= $1;	
}
;

hieroglyphe     : grammaireopt signe modificateurs  options finmotopt
{
	$$= $2;
	$$->estfinmot= $5;
	$$->estgram= $1;	
}								 | HACHUREG
 {
	 $$= new_sign(S_hachureg);
	 $$->type= S_hachureg;
 }
								 | HACHUREH
 {
	 $$= new_sign(S_hachureh);
}
								| HACHUREV
{
	$$= new_sign(S_hachurev);
}
								| HACHURET
{
	$$= new_sign(S_hachuret);
}
;

grammaireopt:   GRAMMAIRE {$$= TRUE;}
                | /* empty */ {$$= FALSE;}
;

finmotopt:      FINMOT {$$= TRUE;}
                | /* empty */ {$$= FALSE;}
;

modificateurs   : /* empty */ {$$ = $<signe>0;} /* le signe doit etre repete */
                | modificateur modificateurs {$$ = $<signe>0;} /* le signe doit etre repete */
;

modificateur    :
SIZE {$$ = $<signe>0; 	$$->size= $1;} /* le signe */
| ROTATION {$$ = $<signe>0; $$->rotation= $1;}
| NROTATION {$$ = $<signe>0; $$->rotation= $1; $$->revert= TRUE;}
| REVERT   {$$ = $<signe>0; 
						$$->revert= TRUE;}
| IGNORE   {$$ = $<signe>0; $$->ignore= TRUE;}
| LONGSIGN {$$ = $<signe>0; $$->islong= $1;}
;

construction_parenthesee :
DSTSUPERFETATOIRE embeddedQuadras FSTSUPERFETATOIRE
{
	$$= new_quadra(S_superfetatoire);						
	$$->contenu.des_quadras= $2;
	$$->kerning= 0;	
}
| DSTEFFACE embeddedQuadras FSTEFFACE
{
	$$= new_quadra(S_efface);						
	$$->contenu.des_quadras= $2;
	$$->kerning= 0;
}
| DSTDISPARU embeddedQuadras FSTDISPARU
{
	$$= new_quadra(S_disparu);						
	$$->contenu.des_quadras= $2;
	$$->kerning= 0;
}
| DSTRAJOUTSCRIBE embeddedQuadras FSTRAJOUTSCRIBE
{
	$$= new_quadra(S_ajoutscribe);
	$$->contenu.des_quadras= $2;
	$$->kerning= 0;
}
| DSTRAJOUTAUTEUR embeddedQuadras FSTRAJOUTAUTEUR
{
	$$= new_quadra(S_ajoutauteur);						
	$$->contenu.des_quadras= $2;
	$$->kerning= 0;
}
;

%%


/**
 *
 */

PUBLIC Texte *
sesh_parse_array(char *array)
{
	extern int seshnumligne;
	extern void SESH_init_seshlex();
	resultat= NULL;
	erreur_de_syntaxe= 0;
	seshnumligne= 0;
	color= 0;
	ingray= FALSE;
	init_array_read(array);
	SESH_init_seshlex();
	yyparse();
	return resultat;
}

/**
	Used for error messages
 */

PUBLIC void 
sesherror(char *s)
{
		extern char *seshtext;
		extern int seshnumligne;
		erreur_de_syntaxe= 1;
		sprintf(rapport_erreur_de_syntaxe, "%s, pres de %s ligne %d\n", s, 
						seshtext, seshnumligne);
}

/*
 * Local Variables:
 * header: "seshaux.h"
 * tab-width: 2
 * End:
 */
