
#define _POSIX_SOURCE 1


/*
	 Interface graphique de TKSESH.
	 Il suffira en thorie de la r-crire pour obtenir
	 une sortie diffrente (exemple intressant : 
	 criture dans un fichier.)
*/

#include "chaine.h"
#include "tgraph.h"
#include "tgraphImpl.h"
#include "general.h"
#include <math.h>

#ifndef _GNUC_
#define inline
#endif

#ifndef NEW
#define NEW(x) (x *)ALLOUER(sizeof(x))
#endif

/**
	 TKSO_get_window --
	 get drawing window coordinates
	*/

TKSESH_rectangle
TKSO_get_window(TKSESH_output *o)
{
	return o->window;
}

/**
	 TKSO_set_window --
	 Sets drawing window coordinates.
	 w and h are pixel dimentions.
	 r is the RxR drawing window
	*/
 
void
TKSO_set_window(TKSESH_output *o, TKSESH_rectangle r, int w, int h)
{
	o->w= w;
	o->h= h;
	o->window= r;
}


double
TKSO_get_real_width(TKSESH_output *o)
{
	return o->window.top_right.x- o->window.bottom_left.x;
}



void 
TKSO_set_scale(TKSESH_output *o, double scale)
{
	o->scale = scale;
}



PUBLIC void
TKSO_toggle_reverse(TKSESH_output *o)
{
	o->reverse= 1 - o->reverse;
}



PUBLIC void 
TKSO_set_state(TKSESH_output *o, 
												TKSO_state s)
{
	if (o->driver->set_stateP != NULL)
		o->driver->set_stateP(o,s);		
	o->state=s;
}



TKSO_state 
TKSO_get_state(TKSESH_output *o)
{
	return o->state;
}


void 
TKSO_set_lineWidth(TKSESH_output *o, 
									 TKSO_lineWidth wd)
{
	o->lineWidth= wd;
}

TKSO_lineWidth
TKSO_get_lineWidth(TKSESH_output *o)
{
	return o->lineWidth;
}


TKSESH_point
TKSO_init_point(double x, double y)
{
	TKSESH_point m;
	m.x= x;
	m.y= y;
	return m;
}


void 
TKSO_genericHachureP(TKSESH_output * o, TKSESH_rectangle rect)
{
	TKSESH_point a, b;
	double range, firsta;
	int n;
	/* the idea is the following. a glyph is *scale* units high.
	 * so we should draw, say, four lines for a sign's height.
	 */
	
	/* we scan the left side of the rectangle */
	/* find the first line to draw */
	/* while the starting point is >= rect.bottom_left.y, draw. */
	range= 1 / 8.0;
	/* the shading lines are : y=x, y=x-range, y= x-2*range, etc... */
	/* we find the first shading line. */
	/* let x0,y0 be the upper left corner.
	 * we look for the biggest n such that :
	 * y0 >= x0 +n*range.
	 * That is y0-x0 >= n*range
	 */
		 
	a.x= rect.bottom_left.x;
	a.y= rect.top_right.y;
	/* compute n */
	n= floor((a.y - a.x)/range);
	
	firsta= n*range+a.x;
	
	while (firsta >= a.y) 
		{
			firsta= firsta - range;
			n--;
		}
	
	a.y= firsta;
	
	while (a.y >= rect.bottom_left.y)
		{
			/* compute b */
			b.y= rect.top_right.y;
			b.x= b.y- n*range;
			if (b.x > rect.top_right.x) {
				b.x= rect.top_right.x;
				b.y= b.x + n*range;
			}
			TKSO_DRAWLINE(o, a, b);
			a.y= a.y - range;
			n--;
		}
	
	
	/* now, scan the bottom side. The idea is the same. */
	a.x= rect.bottom_left.x;
	a.y= rect.bottom_left.y;
	
	n= floor((a.x - a.y)/range);
	firsta= n*range+a.y;
	
	while (firsta <= a.x) 
		{
			firsta= firsta + range;
			n++;
		}
	
	a.x= firsta;


	/* while the starting point is <= rect.top_right.x, draw. */
	while (a.x <= rect.top_right.x)
		{
			/* compute b */
			b.x= rect.top_right.x;
			b.y= b.x- n*range;	
			if (b.y >  rect.top_right.y)
				{
					b.y=  rect.top_right.y;
					b.x= b.y + n*range;
				}
			TKSO_DRAWLINE(o, a, b);
			a.x= a.x + range;
			n++;
		}
}



TKSESH_rectangle
TKSO_init_rectangle(double llx, double lly, double urx, double ury)
{
	TKSESH_rectangle r;
	r.bottom_left.x= llx;
	r.bottom_left.y= lly;
	r.top_right.x= urx;
	r.top_right.y= ury;
	/*
		r.llx= llx;
		r.lly= lly;
		r.urx= urx;
		r.ury= ury;
	*/
	return r;
}


XPoint
TKSESH_point_to_XPoint(TKSESH_output *o, TKSESH_point p)
{
	XPoint m;
	
	/* TKSESH_transformation *tr= o->trans;
		 for (i= 0; i< o->nbtrans; i++)
		 p= TKSESH_apply_transformation(&p, tr[i], p);
		 */
	m.x= (p.x - o->window.bottom_left.x)*o->scale;
	m.y= (o->window.top_right.y - p.y)*o->scale;
	return m;
}


TKSESH_point
XPoint_to_TKSESH_point(TKSESH_output *o, XPoint m)
{
	TKSESH_point p;
	TKSESH_point mm;
	mm.x= m.x;
	mm.y= m.y;
	p.x= (o->window.bottom_left.x+ (mm.x/ o->scale));
	p.y= (o->window.top_right.y- (mm.y/ o->scale));
	return p;
}

void
TKSO_clean(TKSESH_output *o)
{
	o->driver->cleanP(o);
}

void
TKSO_DrawLine(TKSESH_output *o,
								TKSESH_point a,
								TKSESH_point b)
{
	o->driver->DrawLineP(o,a,b);
}

void
TKSO_DrawLines(TKSESH_output *o,
								 int npoints, 
								 TKSESH_point * lespoints)
{
	o->driver->DrawLinesP(o,npoints,lespoints);
}

void
TKSO_hachure(TKSESH_output *o,
							 TKSESH_rectangle rect)
{
	o->driver->hachureP(o, rect);
}

void
TKSO_fillPolygon(TKSESH_output *o,
								 int npoints, 
								 TKSESH_point * lespoints,
								 int sens)
{
		o->driver->fillPolygonP(o, npoints, lespoints, sens);
}

void
TKSO_DrawArc(TKSESH_output *o,
						 TKSESH_point center,
						 double width, double height,
						 int angle1, int angle2)
{
		o->driver->DrawArcP(o, center, width, height, angle1, angle2);
}


double
TKSO_String_width(TKSESH_output *o,
										char *texte)
{
	return o->driver->String_widthP(o, texte);
}

void
TKSO_DrawString(TKSESH_output *o,
								TKSESH_point ou,
								char *texte)
{
	o->driver->DrawStringP(o, ou, texte);
}


void
TKSO_fill_rectangle(TKSESH_output *o, TKSESH_rectangle r)
{
	o->driver->fill_rectangleP(o, r);
}

/*
	TKSO_init_plain --
	General initialization function.
	For use by device drivers ONLY
*/

void
TKSO_init_plain(TKSESH_output *o, double x1, double y1, double x2, double y2, int w, int h)
{
	o->driver= 0;
	o->reverse= 0;
	o->window= TKSO_init_rectangle(x1,y1,x2,y2);
	o->w= w;
	o->h= h;
	o->state= black;
	o->lineWidth= nullWidth;
	o->isLowRes= FALSE;
	o->hasSplines= FALSE;
	o->useFonts= FALSE;
	/* o->scale= 14.0;*/
	o->scale= o->w/(x2-x1);
}

int
TKSO_isLowRes(TKSESH_output *o)
{
	return o->isLowRes;
}

/* Let's preserve the present presentation
 * Local Variables:
 * tab-width: 2
 * c-block-comments-indent-p: nil
 * End:
 */
