# Class representing a BZR font. See BZRFont.old for a Tcl only
# version.  As Tcl-only was a bit slow even with an AMD 900MH, whe
# chose to keep using a C library.
#
# 
# Note on BZR format :
#	1) size (Bdouble)
#	2) 
#
# Codes : see file bzrop.h in the src directory
#
# Note on Tcl Format for a given char :
# 	a list with : code charwidth llx lly urx ury contours*
#	a contour :
#	"CMP" sens firstPoint segments*
#	
#	spline segment list : s c1 c2 p2 
#	line segment list : l p2

package provide BZRFonts 1.0

package require Sbzr 2.0
package require BzrToType1

class BZRFont {    
    
    proc BZRFont {this} {
	set ($this,fontfile) {None}
	#set ($this,font) [sbzr_newfont]
	setFontSize $this 14
	setFileName $this "unnamed.bzr"
	setName $this "BZR edited font"
	clearFont $this
    }

    proc ~BZRFont {this} {
    }

    proc load {this fileName} {
	clearFont $this
	set txt [sbzr_loadfont $fileName]
	setFontSize $this [lindex $txt 0]
	setName $this [lindex $txt 1]
	setFileName $this $fileName
	# index 2 == font bounding box.
	foreach ch [lrange $txt 3 end] {
	    # We set the char, without fixing the path
	    # orientation, because it's already good.
	    setChar $this [lindex $ch 0] $ch 0
	}
    }

    proc saveAs {this fname} {
	sbzr_savefont [getAsList $this] $fname
    }

    proc getBoundingBox {this} {
	set bllx 1000
	set blly 1000
	set burx -1000
	set bury -1000
	for {set i 0} {$i < 128} {incr i} {
	    if {[hasChar $this $i]} {
		set rep [getChar $this $i]
		set llx [lindex $rep 2]
		set lly [lindex $rep 3]
		set urx [lindex $rep 4]
		set ury [lindex $rep 5]
		if {$llx < $bllx} {
		    set bllx $llx
		}
		if {$lly < $blly} {
		    set blly $lly
		}
		if {$urx > $burx} {
		    set burx $urx
		}
		if {$ury > $bury} {
		    set bury $ury
		}
	    }   
	}
	return [list $bllx $blly $burx $bury]
    }
    
    proc getAsList {this} {
	set txt [list [getFontSize $this] [getName $this] [getBoundingBox $this]]
	for {set i 0} {$i < 256} {incr i} {
	    if {[hasChar $this $i]} {
		lappend txt [getChar $this $i]
	    }
	}
	return $txt
    }

    # Ensures a char's data are correct, and in particular that positive contours
    # turn the right way
    
    proc fixChar {txt} {
	set res [lrange $txt 0 5]
	# fix each path
	foreach p [lrange $txt 6 end] {
	    set orient [lindex $p 1]
	    set w [sbzr_pathwinding $p]
	    if {([string equal $orient P] && $w < 0)
	    || ([string equal $orient N] && $w > 0)} {
		puts "Bad reverse for path $p"
		# We need to reverse the path
		set newp [lrange $p 0 3]
		# for example : p0 {s c1 c2 p1} {s c3 c4 p2} {t p0}
		# will become : p0 {t p2} {s c4 c3 p1} {s c2 c1 p0}
		# So, to build an element, we need to know the first
		# point of the next one.

		# index of the last element :
		set l [expr {[llength $p] - 1}]
		for {set i $l} {$i > 3} {incr i -1} {
		    # last point of previous element :
		    if {$i > 4} {
			set e [lindex $p [expr {$i - 1}]]
			set y0 [lindex $e end]
			set x0 [lindex $e end-1]
		    } else {
			set x0 [lindex $p 2]
			set y0 [lindex $p 3]
		    }
		    # Now, we build the new segment :
		    set old [lindex $p $i]
		    set seg [list [lindex $old 0]]
		    # index of y of the first point in the new segment :
		    set d [expr {[llength $old] - 3}]
		    for {set j $d} {$j > 0} {incr j -2} {
			set y [lindex $old $j]
			set x [lindex $old [expr {$j - 1}]]
			lappend seg $x
			lappend seg $y
		    }
		    lappend seg $x0 $y0
		    # The segment is complete.
		    # we add it to the new path
		    lappend newp $seg
		}
		lappend res $newp
	    } else {
		lappend res $p
	    }
	}
	return $res;
    }

    proc exportAsType1 {this fname} {
	set l [getAsList $this]
	# set name [getName $this]
	set base [file rootname [file tail $fname]]
	set t [BzrToType1::bzrToType1 $l $base]
	set f [open $fname w]
	puts $f $t
	close $f
	set f [open "[file rootname $fname].afm" w]
	puts $f [BzrToType1::bzrToAFM $l $base]
	close $f
    }
    
    proc exportAsMF {this mfName} {
	# export in two stages :
	# a create a wrapper mf file
	set wrapper {
% This file defines the Metafont font FILENAME
true_design_size# := 14.000pt#;
input Hibzrstp;
font_coding_scheme := "Egyptian font FILENAME";
font_identifier := "FILENAME";
input FILENAME_font.mf;

def mamodif= identity  reflectedabout((w/2,h), (w/2,0)); enddef;
def depart= 128; enddef;
input FILENAME_font.mf;
bye.
	}
	
	# compute actual file names
	set base [file rootname [file tail $mfName]]
	set wrapper [string map [list FILENAME $base] $wrapper]
	
	set f [open $mfName w]
	puts $f $wrapper
	close $f
	
	set auxFileName "[file rootname $mfName]_font.mf"
	set f [open $auxFileName w]
	# b create the real font file :
	# produce each sign.
	for {set i 0} {$i < 128} {incr i} {
	    if {[hasChar $this $i]} {
		set rep [getChar $this $i]
		set code [lindex $rep 0]
		set width [lindex $rep 1]
		set llx [lindex $rep 2]
		set lly [lindex $rep 3]
		set urx [lindex $rep 4]
		set ury [lindex $rep 5]

		puts $f "beginchar ($i, ${width}u#, [expr $ury - $lly]u#, 0.000u#);"
		puts $f "\"$base$i\";"
		# list the char control points
		set cId 0
		foreach c [lrange $rep 6 end] {
		    set pId 0
		    set wind [lindex $c 1]
		    set xs [lindex $c 2]
		    set ys [lindex $c 3]
		    puts $f "\tz$i\\$cId\\${pId}s = (${xs}u, ${ys}u);"
		    foreach s [lrange $c 4 end] {
			switch -exact -- [lindex $s 0] {
			    s {
				puts $f "\tz$i\\$cId\\${pId}c1 = ([lindex $s 1]u, [lindex $s 2]u);"
				puts $f "\tz$i\\$cId\\${pId}c2 = ([lindex $s 3]u, [lindex $s 4]u);"
				puts $f "\tz$i\\$cId\\${pId} = ([lindex $s 5]u, [lindex $s 6]u);"
			    }
			    l {
				puts $f "\tz$i\\$cId\\${pId} = ([lindex $s 1]u, [lindex $s 2]u);"
			    }
			}
			incr pId			
		    }
		    incr cId
		}
		# Build the char's contours
		set cId 0
		foreach c [lrange $rep 6 end] {
		    set pId 0
		    set wind [lindex $c 1]
		    puts $f "\tfill_or_unfill z$i\\$cId\\0s"
		    set lastSegmentId [expr [llength $c] - 5]
		    foreach s [lrange $c 4 end] {
			switch -exact -- [lindex $s 0] {
			    s {
				puts -nonewline $f "\t\t..controls z$i\\$cId\\${pId}c1 and "
				puts -nonewline $f "z$i\\$cId\\${pId}c2.."
			    }
			    l {
				puts -nonewline $f "\t\t--"
			    }
			}
			if {$pId == $lastSegmentId} {
			    puts $f "cycle;"
			} else {
			    puts $f "z$i\\$cId\\${pId}"
			}
			incr pId
		    }
		    incr cId
		}
		puts $f "endchar;"
	    }
	}
	close $f
    }

    # Public functions
    
    proc setFileName {this fileName} {
	set ($this,fileName) $fileName
    }

    proc getFileName {this} {
	return $($this,fileName)
    }

    proc setName {this name} {
	set ($this,name) $name
    }

    proc getName {this} {
	return $($this,name)
    }

    proc setFontSize {this size} {
	set ($this,size) $size
    }

    proc getFontSize {this} {
	return $($this,size)
    }

    # set a char's value.
    # fix : if fix is true, we will ensure the winding pathes are
    # correct.
    
    proc setChar {this code txt {fix 1}} {
	if {$fix} {
	    set txt [fixChar $txt]
	}
	set ($this,char,$code) $txt
	setHasChar $this $code 1
    }

    proc hasChar {this code} {
	return $($this,hasChar,$code)
    }

    proc setHasChar {this code has} {
	set ($this,hasChar,$code) $has
    }

    proc getChar {this code} {
	if {[hasChar $this $code]} {
	    set res $($this,char,$code)
	} else {
	    set res [list $code 0 0 0 0 0]
	}
	return $res
    }
    
    proc clearChar {this code} {
	set ($this,char,$code) [list]
	setHasChar $this $code 0
    }

    proc clearFont {this} {
	for {set i 0} {$i < 256} {incr i} {
	    clearChar $this $i
	}
    }
}
