# This class regulates the communication between UI and 
# the DB database.

# Messages sent to the update function :
# What the observers want is to know if there is a new glyph to display or not

package require TkseshDB
#package require seshModel
package require glyphCode
package require glyphDAL
class CharDBWorkflow {

    proc CharDBWorkflow {this tkseshdb} Observable {} Observer {} {
	set ($this,tkseshdb) $tkseshdb
	
	# DataSources :
	#	sign	
	#	values for this sign

	set ($this,glyphDS) [TkseshDB::buildTableDataSource $tkseshdb \
				 "GlyphView" {uid mid id repOid}\
				 {uid i mid i id i repOid i date t sid i 
				     drawing t guides t}]

	# After testing, the gain from not loading everything in
	# memory is minor. 
	# DataSource::setResultKeptInMemory $($this,glyphDS) 0

	# In fact, we want to control glyphDS using the signs code. So we change
	# the select request (the other ones are fine)
	
	# The following request can be expanded by stuff like 'and code glob "A*"'
	DataSource::setSelectRequest $($this,glyphDS) {
	    select distinct uid, mid, id, repOid, date, sid, drawing, guides
	    from GlyphView, HasCode
	    where GlyphView.repOid = HasCode.glyphId
	}

	DataSource::setPrepareValidate $($this,glyphDS) \
	    "TkseshDB::prepareUserEntry $tkseshdb Glyph"
	# DataSource::setDefault $($this,glyphDS) date [TkseshDB::getDate $tkseshdb]

	# DataSource : code(s) for this glyph :
	set ($this,codeDS) [TkseshDB::buildTableDataSource $tkseshdb \
				 "HasCodeView" {uid mid id repOid}\
				 {uid i mid i id i repOid i date t sid i 
				     glyphId i code t isOfficial i}]
	DataSource::declareExternal $($this,codeDS) {currentID i}
	DataSource::setExternalCol $($this,codeDS) currentID 0
	DataSource::setDefault $($this,codeDS) code "???"
	DataSource::setDefault $($this,codeDS) isOfficial 0	
	DataSource::setPrepareValidate $($this,codeDS) \
	    "TkseshDB::prepareUserEntry $tkseshdb HasCode"
	#
	DataSource::setFilter $($this,codeDS) "where glyphId = :currentID:"
	DataSource::setDeletableTest $($this,codeDS) "CharDBWorkflow::cantChangeOfficialCodeTest $this"
	DataSource::setEditableTest $($this,codeDS) "CharDBWorkflow::cantChangeOfficialCodeTest $this"
	# glyphId will take its value from currentID when *creating* a new record.
	DataSource::bindFields $($this,codeDS) {glyphId currentID}
	
	# DataSource : values for this glyph :
	set ($this,valueDS) [TkseshDB::buildTableDataSource $tkseshdb \
				 "HasValueView" {uid mid id repOid} \
				 {uid i mid i id i repOid i date t sid i
				     code t value t valueType t 
				     usualValue t usualSign t period t
				 }]
	
	DataSource::declareExternal $($this,valueDS) {currentCode t}
	DataSource::setExternalCol  $($this,valueDS) currentCode 0
	# defaults ...
	DataSource::setDefault $($this,valueDS) value "???"
	DataSource::setDefault $($this,valueDS) valueType "X"
	DataSource::setDefault $($this,valueDS) usualValue "C"
	DataSource::setDefault $($this,valueDS) usualSign "C"
	DataSource::setDefault $($this,valueDS) period "0"
	#
	DataSource::setPrepareValidate $($this,valueDS) \
	    "TkseshDB::prepareUserEntry $tkseshdb HasValue"
	DataSource::setFilter $($this,valueDS) "where code = :currentCode:"
	DataSource::bindFields $($this,valueDS) {code currentCode}

	# we listen to glyphDS, to update currentID.
	Observable::add $($this,glyphDS) $this
	# we listen to codeDS to update valueDS
	Observable::add $($this,codeDS) $this	
	setFilter $this "*"
    }

    proc ~CharDBWorkflow {this} {
    }

    proc getCodeDS {this} {
	return $($this,codeDS)
    }

    proc getValueDS {this} {
	return $($this,valueDS)
    }

    proc getGlyphDS {this} {
	return $($this,glyphDS)
    }
    
    proc getFilter {this} {
	return $($this,filter)
    }
    
    
    proc setFilter {this filter} {
	set ($this,filter) $filter
	if {$filter eq "*" || $filter eq ""} {
	    DataSource::setFilter $($this,glyphDS) ""
	} elseif {[string first * $filter] != -1} {
	    DataSource::setFilter $($this,glyphDS) "and code GLOB '$filter'"
	} else {
	    DataSource::setFilter $($this,glyphDS) "and code = '$filter'"
	}
	#update $this xx xx
    }

    proc update {this orig msg} {
	#puts "updating CharDBWorkflow"
	# Ensure we display the correct entry
	catch {
	    if {$orig == $($this,glyphDS)} {
		#puts "changement de signe"
		if {[DataSource::hasRow $($this,glyphDS)]} {
		    set i [DataSource::getCol $($this,glyphDS) repOid]
		    DataSource::setExternalCol $($this,codeDS) currentID $i
		    Observable::notify $this ROWCHANGED
		}
	    } elseif {$orig == $($this,codeDS)} {
		#puts "changement de code"
		if {[DataSource::hasRow $($this,codeDS)]} {
		    set i [DataSource::getCol $($this,codeDS) code]
		    DataSource::setExternalCol $($this,valueDS) currentCode $i
		    Observable::notify $this ROWCHANGED
		}
	    }
	}
    }

    proc getCurrentGlyphCode {this} {
	set ds $($this,glyphDS)
	set uid [DataSource::getCol $ds uid]
	set mid [DataSource::getCol $ds mid]
	set id [DataSource::getCol $ds id]
	return [glyphCode::buildCode $uid $mid $id]
    }
    
    proc getCurrentSignDrawing {this} {
	return [DataSource::getCol $($this,glyphDS) drawing]
    }
    
    # The glyph naming system is now in the Glyph class. It shouldn't.
    # Potential system weakness.
    proc getNewGlyphLabel {this} {
	set d $($this,tkseshdb)	
	set uid [TkseshDB::getUid $d]
	set mid [TkseshDB::getMid $d]
	return [glyphCode::buildNewGlyphLabel $uid $mid]
	return "UG${uid}M${mid}N??"
    }
    
    
    proc hasGlyph {this} {
	return [DataSource::hasRow $($this,glyphDS)]
    }

    # Save char GlyphCode. If the said char already exists, it will
    # be modified. Else, it will be created.


    proc setChar {this glyphLabel drawing} {

	glyphCode::getUserGlyphDataFromCode $glyphLabel uid mid id
	
	set d $($this,tkseshdb)
	set g $($this,glyphDS)
	
	TkseshDB::begin $d
	
	#puts "A"
	if {[string equal $id "??"]} {
	    DataSource::createRow $g
	    DataSource::setCol $g drawing $drawing
	    DataSource::setCol $g guides {}
	    # data is fixed by prepareUserEntry
	    #puts "Official prep. call"
	    TkseshDB::prepareUserEntry $d "Glyph" $g
	    DataSource::validation $g
	    # get the glyph label 
	    setFilter $this [TkseshDB::getLastGlyphCode $d]
	} else {
	    # if the current char is not the validated one :
	    if {[getCurrentGlyphCode $this] ne "$glyphLabel"} {
		# we restrict the search to this sign :
		setFilter $this "$glyphLabel"
	    }
	    # Refresh is automatic
	    DataSource::setCol $g drawing $drawing
	    DataSource::setCol $g guides {}
	    DataSource::validate $g
	}
	
	TkseshDB::commit $d

	Observable::notify $this NEWSIGN
    }

    
    proc testCanInsert {this dsSign} {
	set uid [TkseshDB::getUid $d]
	catch {
	    if {$uid == [DataSource::getCol $dsSign "uid"]} {
		return 1
	    }
	}
	return 0
    }
    
    proc cantChangeOfficialCodeTest {this ds} {
	set isOfficial [DataSource::getCol $ds isOfficial]
	if {$isOfficial} {
	    return 0
	} else {
	    return 1
	}
    }

    # Return a string suitable for char exportation into a file.

    proc exportCurrentChar {this} {
	set db $($this,tkseshdb)
	set glyphId [DataSource::getCol [getGlyphDS $this] repOid]
	set uid [TkseshDB::getUid $db]

	set res [list]
	set code [getCurrentGlyphCode $this]
	
	set sql "select uid, mid, id, date, sid, drawing, guides\
                 from GlyphView where repOid = $glyphId"
	lappend res [concat Glyph [TkseshDB::select $db $sql]]
	
	
	foreach c [exportCodesForGlyph $this $code $glyphId] {
	    lappend res $c
	}

	# values are not yet saved. We wait to define them better.
	return $res
    }

    proc exportCodesForGlyph {this code glyphId} {
	set db $($this,tkseshdb)
	set uid [TkseshDB::getUid $db]
	
	# compute the codes. Only codes belonging to the current user are saved.
	set sql \
	    "select HasCodeView.uid as uid, HasCodeView.mid as mid, HasCodeView.id as id,\
                    HasCodeView.date as date, HasCodeView.sid as sid, \
                    gl.uid as guid, gl.mid as gmid, gl.id as gid, \
                    code, isOfficial\
             from HasCodeView, Header as gl \
             where glyphId = $glyphId \
             and gl.type= 'Glyph'\
             and HasCodeView.glyphId = gl.repOid\
             and code != '$code' \
             and HasCodeView.uid == $uid"
	
	set res [list]
	TkseshDB::doSelect $db $sql tmp {
	    lappend res [list HasCode \
			     $tmp(uid) $tmp(mid) $tmp(id) $tmp(date) $tmp(sid)\
			     $tmp(guid) $tmp(gmid) $tmp(gid) $tmp(code) $tmp(isOfficial)]
	    puts $res
	}
	return $res
    }
    
    proc exportUserChars {this} {
	set db $($this,tkseshdb)
	set uid [TkseshDB::getUid $db]
	set res [list]
	set sql "select uid, mid, id, repOid, date, sid, drawing, guides  \
                 from GlyphView where uid = $uid"
	TkseshDB::doSelect $db $sql tmp {
	    lappend res [list Glyph \
			     $tmp(uid) $tmp(mid) $tmp(id) $tmp(date) $tmp(sid)\
			     $tmp(drawing) $tmp(guides)]
	    # Official user glyph code. Not saved.
	    set code [glyphCode::buildCode $tmp(uid) $tmp(mid) $tmp(id)]
	    foreach c [exportCodesForGlyph $this $code $tmp(repOid)] {
		lappend res $c
	    }
	}
	return $res
    }

    # Import a string for char importation.
    # Basically : chartxt can be read as a list : 
    # [list $code $drawing $codes $values]
    # where $code is the user glyph code for the sign and $drawing is the sign drawing
    # codes is a list of auxiliary code.
    # $values is a list of values.
    #
    # return : the char's user's code

    proc importChar {this lst} {
	set db $($this,tkseshdb)
	set ucode ""
	foreach item $lst {
	    switch -- [lindex $item 0] {
		Glyph {
		    foreach {uid mid id date sid drawing guides} [lrange $item 1 end] {
			set date [TkseshDB::protectText $date]
			set drawing [TkseshDB::protectText $drawing]
			set guides [TkseshDB::protectText $guides]
			set sql "insert or replace into GlyphView 
                                 (uid, mid, id, date, sid, drawing, guides)
                                 values ($uid, $mid, $id, '$date', $sid, 
                                         '$drawing', '$guides')"
			#puts "should do $sql"
			TkseshDB::update $db $sql
			set ucode [glyphCode::buildCode $uid $mid $id]
		    }
		}
		HasCode {
		    foreach {uid mid id date sid
			gluid glmid glid code isOfficial} [lrange $item 1 end] {
			set date [TkseshDB::protectText $date]
			set code [TkseshDB::protectText $code]
			set glyphId [glyphDAL::getGlyphIdFor $gluid $glmid $glid]
			set sql "insert or replace into HasCodeView 
                                 (uid, mid, id, date, sid, glyphId, code, isOfficial)
                                 values ($uid, $mid, $id, '$date', 
                                         $sid, $glyphId, '$code', $isOfficial)"
			#puts "should do $sql"
			TkseshDB::update $db $sql
			set ucode $code
		    }
		}
	    }
	}
	return $ucode
    }
}


