# Defines toolbars for the font editor application
# A toolbar controls an editor application.


class CharEditToolBar {
    
    # Describe and define the state buttons.
    # fields : button text, button help, state controled by the button.
    set (stateButtons) {
	"Edit" "Edit, move contours, etc." edit
	"Add"  "Create new contours" add
	"Delete" "Delete stuff" delete
	"Add Guide" "Add guide lines" addguide
	"spline <-> Line" "transform splines into lines and vz." splinetoline
	"select" "select" select
	"lower" "lowers a contour" lower
	"split" "adds a control point on a curve" split
	"reverse" "transforms a positive contour into a negative one and vz." reverse
	"Hor. refl." "Horizontal reflection." hreflect
	"Vert. refl." "Vertical reflection." vreflect
	"Rotate" "Rotate selected." rotate
    }
    
    proc CharEditToolBar {this parent} {

	# Counter for separators
	set ($this,sepNum) 0

	set ($this,parent) $parent
	# set w [Component::getPath $parent]
	
	set bar0 [SMainFrame::addToolBar $parent SFrame]

	set tb [Component::getPath $bar0]

	# In bbox1, we put buttons for selecting drawing colour.
	
	set bbox1 [frame $tb.bbox1]
	
	radiobutton $bbox1.black -indicatoron 0 -background black -width 1 \
		-selectcolor black -value "P" -borderwidth 5 \
		-variable CharEditToolBar::($this,fill) \
		-command [list CharEditToolBar::setBlack $this]
	
	DynamicHelp::register $bbox1.black balloon "Created contours will be filled in black" 

	radiobutton $bbox1.white -indicatoron 0 -background white -width 1 \
		-selectcolor white -value "N" -borderwidth 5 \
		-variable CharEditToolBar::($this,fill) \
		-command [list CharEditToolBar::setWhite $this]

	DynamicHelp::register $bbox1.white balloon "Created contours will be filled in white" 

	radiobutton $bbox1.nofill -indicatoron 0 -background white -width 1 \
		-selectcolor white -value "S" -borderwidth 5 \
		-variable CharEditToolBar::($this,fill) \
		-command [list CharEditToolBar::setNoFill $this]

	DynamicHelp::register $bbox1.nofill balloon "Created contours will be drawn as an unfilled closed curve" 

	radiobutton $bbox1.openCurve -indicatoron 0 -background white -width 1 \
		-selectcolor white -value "T" -borderwidth 5 \
		-variable CharEditToolBar::($this,fill) \
		-command [list CharEditToolBar::setOpenCurve $this]

	DynamicHelp::register $bbox1.openCurve balloon "Created contours will be drawn as an open curve" 

	#pack $bbox1.black $bbox1.white $bbox1.nofill $bbox1.openCurve -side left 
	
	# For a while, only use these two :
	pack $bbox1.black $bbox1.white  -side left 

	SMainFrame::showToolBar $parent $bar0 1

	pack $bbox1 -side left

	addSeparator $this $bar0

	# select operation.
	# will be linked automagically to the variables.

	set bbox2 [frame $tb.bbox2]
	set i 0
	foreach {btext bhelp bstate} $(stateButtons) {
	    set mypath $bbox2.button$i
	    incr i

	    setStateButtonPath $this $bstate $mypath
	    radiobutton $mypath -indicatoron 0  -value $bstate -text $btext 
	    DynamicHelp::register $mypath balloon  $bhelp
	    pack $mypath -side left 
	}
	pack $bbox2 -side left
	addSeparator $this $bar0

	set bar1 [SMainFrame::addToolBar $parent SFrame]
	set tb1 [Component::getPath $bar1]
	
	set scaler \
		[scale $tb1.scale -from -10  -to 10 \
		-tickinterval 5.0 -resolution 0.1 -length 200\
		-command [list CharEditToolBar::scaleEditor $this] -orient horizontal]

	pack $scaler -side left
	setScaler $this $scaler
	addSeparator $this $bar1
	SMainFrame::showToolBar $parent $bar1 1

	setEditor $this 0
    }

    proc ~CharEditToolBar {this} {
    }

    
    proc setScaler {this s} {
	set ($this,scaler) $s
    }

    proc getScaler {this} {
	return $($this,scaler)
    }
    
    proc addSeparator {this tb} {
	incr ($this,sepNum)
	set path [Component::getPath $tb]
	pack [Separator $path.sep$($this,sepNum) -orient vertical] -side left -fill y -padx 4
    }

    proc setEditor {this e} {
	set ($this,editor) $e
	bindStates $this
    }

    proc setStateButtonPath {this state path} {
	set ($this,statepath,$state) $path
    }

    proc getStateButtonPath {this state} {
	return $($this,statepath,$state)
    }

    proc bindStates {this} {
	
	set editor [getEditor $this]
	# Links the states with an actual editor

	if {$editor != 0} {
	    set var CharEdit::($editor,state)
	    CharEdit::abortActions $editor
	    
	    foreach {btext bhelp bstate} $(stateButtons) {
		set p [getStateButtonPath $this $bstate]
		set command [list CharEdit::abortActions $editor]
		$p configure -variable $var -command $command
	    }
	    set s [CharEdit::getScale $editor]
	    [getScaler $this] set $s
	    set ($this,fill) [CharEdit::getFill $editor]
	}
    }


    # Should probably move to apply
    proc getEditor {this} {
	return $($this,editor)
    }
    
    proc hasEditor {this} {
	if {[getEditor $this]  != 0} {
	    return 1
	} else {
	    return 0
	}

    }

    # Auxiliary actions for functions.
    # values will be retrieved and set through setEditor and getEditor.
    
    proc setWhite {this} {
	if {[hasEditor $this]} {
	    CharEdit::setUnFilled [getEditor $this]
	}
    }

    proc setBlack {this} {
	if {[hasEditor $this]} {
	    CharEdit::setFilled [getEditor $this]
	}
    }

    proc setNoFill {this} {
	if {[hasEditor $this]} {
	    CharEdit::setNoFill [getEditor $this]
	}
    }

    proc setOpenCurve {this} {
	if {[hasEditor $this]} {
	    CharEdit::setOpenCurve [getEditor $this]
	}
    }
    
    proc scaleEditor {this scale} {
	if {[hasEditor $this]} {
	    CharEdit::scaleall [getEditor $this] $scale
	}
    }
}
