
#
# Class for commands
#
#

class MyCommand {
    # MyCommand --
    proc MyCommand {this f} {
	set ($this,owner) $f
    }
    
    proc ~MyCommand {this} {
    }

    proc getCharEdit {this} {
	return $($this,owner)
    }

    virtual proc execute {this}
    
    virtual proc undo {this}
}


class SimpleCommand {
    # SimpleCommand --
    # 	Simple undoable commands
    #
    #	f : the CharEdit widget

    proc SimpleCommand {this f} MyCommand {$f} {
	set ($this,old)  [CharEdit::asText $f]
	set ($this,changed) [CharEdit::getChanged $f]
    }

    proc ~SimpleCommand {this} {}

    proc undo {this} {
	CharEdit::clear [MyCommand::getCharEdit $this]
	CharEdit::setText [MyCommand::getCharEdit $this] $($this,old)
	
    }

    virtual proc execute {this}
    
}

class ContourDeleteCommand {
    proc ContourDeleteCommand {this contour} SimpleCommand {[GraphElt::getCharEdit $contour]} {
	set ($this,contour) $contour
    }

    proc ~ContourDeleteCommand {this} {}

    proc execute {this} {
	delete $($this,contour)
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}

class SegmentSplitCommand {
    proc SegmentSplitCommand {this segment x y} SimpleCommand {[GraphElt::getCharEdit $segment]} {
	set ($this,segment) $segment
	set ($this,x) $x
	set ($this,y) $y
    }
    
    proc  ~SegmentSplitCommand {this} {}

    proc execute {this} {
	Segment::split $($this,segment) $($this,x) $($this,y)
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}


class SplineToLineCommand {
    proc SplineToLineCommand {this segment} SimpleCommand {[GraphElt::getCharEdit $segment]} {
	set ($this,segment) $segment
    }

    proc ~SplineToLineCommand  {this} {}
    proc execute {this} {
	Segment::change $($this,segment)
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}

class LowerCommand {
    proc LowerCommand {this contour} SimpleCommand {[GraphElt::getCharEdit $contour]} {
	set ($this,contour) $contour
    }
    
    
    proc ~LowerCommand {this} {}

    proc execute {this} {
	CharEdit::lowerContour [MyCommand::getCharEdit $this] $($this,contour)
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}



class RaiseCommand {
    proc RaiseCommand {this contour} SimpleCommand {[GraphElt::getCharEdit $contour]} {
	set ($this,contour) $contour
    }
    

    proc ~RaiseCommand {this} {}

    proc execute {this} {
	CharEdit::raiseContour [MyCommand::getCharEdit $this] $($this,contour)
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}


# This class represents an element moving.
# 

class MoveCommand {
    proc MoveCommand {this elt x y state} MyCommand {[GraphElt::getCharEdit $elt]} {
	set ($this,elt) $elt
	set ($this,x) $x
	set ($this,y) $y
	set ($this,old) $state
    }
    

    proc ~MoveCommand {this} {}

    proc execute {this} {
	# The actual move has already occurred
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
    
    proc undo {this} {
	CharEdit::clear [MyCommand::getCharEdit $this]
	CharEdit::setText [MyCommand::getCharEdit $this] $($this,old)
    }
}

class PointDeleteCommand {
    proc PointDeleteCommand {this pt} SimpleCommand {[GraphElt::getCharEdit $pt]} {
	set ($this,pt) $pt
    }

    proc ~PointDeleteCommand {this} {}

    proc execute {this} {
	ControlPoint::deletePoint $($this,pt)
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}


class NewContourCommand {
    proc NewContourCommand {this edit x y type} SimpleCommand {$edit} {
	set ($this,x) $x
	set ($this,y) $y
	set ($this,type) $type
    }

    proc ~NewContourCommand {this} {}

    proc execute {this} {
	CharEdit::ajouteContour [MyCommand::getCharEdit $this] \
		$($this,x) $($this,y) $($this,type)
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}

class PasteCommand {
    proc PasteCommand {this edit txt} SimpleCommand {$edit} {
	set ($this,txt) $txt
    }

    proc ~PasteCommand {this} {}

    proc execute {this} {
	CharEdit::addtext [MyCommand::getCharEdit $this] $($this,txt)
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}

class ContourToggleFillCommand {
    proc ContourToggleFillCommand {this contour} SimpleCommand {[GraphElt::getCharEdit $contour]} {
	set ($this,contour) $contour
    }

    proc ~ContourToggleFillCommand {this} {}

    proc execute {this} {
	if {[string compare [Contour::getFill $($this,contour)] "P"] == 0} {
	    Contour::setFill $($this,contour) "N"
	} else {
	    Contour::setFill $($this,contour) "P"
	}
	CharEdit::setChanged [MyCommand::getCharEdit $this] 1
    }
}
