package require stooop
package provide TkseshDB 1.0
package require tclSQLDB
#package require seshModel
package require seshuser

class TkseshDB {
    set (sgl) 0

    # PRIVATE !!!!
    proc TkseshDB {this db} {
	set ($this,db) $db
	# isOfficial is true if we are working as official database manager.
	set ($this,isOfficial) 0
	# limitToUser is true if searches are limited to the current user's entries
	set ($this,limitToUser) 0
	# are we in a transaction
	set ($this,inTransaction) 0
    }

    proc ~TkseshDB {this} {
	delete $($this,db)
    }

    proc close {this} {
	SQLDB::close $($this,db)
    }

    proc getDBFilePath {} {
	return [file join [seshuser::getvar PATH] seshDB.db]
    }
    
    proc buildDataSource {this sql} {
	return [new DataSource [sql $this] $sql]
    }

    proc buildTableDataSource {this table idx cols} {
	return [DataSource::buildTableDataSource [sql $this] $table $idx $cols]
    }

    proc isOfficial {this} {
	return $($this,isOfficial)
    }

    proc setOfficial {this val} {
	set ($this,isOfficial) $v
    }
    
    # Should move to a DBUtil class or package.
    proc getDate {} {
	set date [clock format [clock seconds] -format "%Y %m %d %H %M %S"]
	return $date
    }

    proc limitToUser {this val} {
	set ($this,limitToUser) $val
    }
    
    proc isLimitedToUser {this} {
	return $($this,limitToUser)
    }

    proc sql {this} {
	return $($this,db)
    }

    proc inTransaction {this} {
	return $($this,inTransaction)
    }

    proc begin {this} {
	set ($this,inTransaction) 1
	SQLDB::update [sql $this] "begin transaction"
    }
    
    proc commit {this} {
	SQLDB::update [sql $this] "commit"
	set ($this,inTransaction) 0
    }

    proc rollback {this} {
	SQLDB::update [sql $this] "rollback"
	set ($this,inTransaction) 0
    }

    proc update {this req} {
	SQLDB::update [sql $this] $req
    }

    proc buildTableDataSource {this tableName key cols} {
	return [DataSource::buildTableDataSource [sql $this]\
		    $tableName $key $cols]
    }

    proc buildDataSource {this sql} {
	return [new DataSource [sql $this] $sql]
    }
 
    proc select {this req} {
	return [[SQLDB::getDBCMD [sql $this]] eval $req]
    }

    proc doSelect {this req array body} {
	#puts "doing $req"
	uplevel [SQLDB::getDBCMD [sql $this]] eval [list $req] $array [list $body]
    }
    
    proc lastRowID {this} {
	return [SQLDB::lastRowID [sql $this]]
    }

    proc getTkseshDB {} {
	if {$(sgl) == 0} {
	    set db [new SQLDB [getDBFilePath]]
	    set (sgl) [new TkseshDB $db]
	}
	return $(sgl)
    }
    
    proc getUid {this} {
	return [seshuser::getUID]
    }

    proc getSid {this} {
	return 1
    }

    proc getMid {this} {
	return [seshuser::getMID]
    }
    
    # Compute the values for a new user entry of a given type.
    # should be called before saving one.
    
    proc prepareUserEntry {this type dataSource} {
	#puts "calling prepareUserEntry $type"
	if {[DataSource::isNull $dataSource id]} {
	    #set id [getNewID $this $type]
	    #puts "id is $id"
	    #DataSource::setCol $dataSource id $id
	}
	if {[DataSource::isNull $dataSource uid]} {
	    set uid [getUid $this] 
	    set mid [getMid $this]
	    #puts "uid $uid mid $mid"
	    DataSource::setCol $dataSource uid $uid
	    DataSource::setCol $dataSource mid $mid
	}
	DataSource::setCol $dataSource "date" [getDate]
	DataSource::setCol $dataSource sid 1
	#puts "B id is NULL : [DataSource::isNull $dataSource id]"	
	#puts "B id has value : [DataSource::getCol $dataSource id]"
    }
 
    # PRIVATE proc :
    proc getNewID {this type} {
	error "should not be used"
	set res [select $this "select maxid from MaxTable where type = '$type'"]
	incr res
	#puts "res is $res"
	update $this "update MaxTable set maxid = $res where type = '$type'"
	return $res
    }

    proc getLastGlyphCode {this} {
	set res [select $this {
	    select code
	    from HasCodeView, MaxTable
	    where glyphId = MaxTable.lastOid
	    and   MaxTable.type = 'Glyph'
	    and isOfficial = 1
	}]
	return [lindex $res 0]
    }
    
    proc getLatestElementID {this type} {
	return [select $this "select lastOid from MaxTable where type = '$type'"]
    }

    proc protectText {text} {
	return [SQLDB::protectText $text]
    }
    
    proc saveUser {this uid mid name mail} {
	set sql {
	    update Management
	    set uid = UID,
	    mid = MID,
	    name = 'NAME',
	    mail = 'MAIL'
	}
	set name [protectText $name]
	set mail [protectText $mail]
	set sql [string map [list UID $uid MID $mid NAME $name MAIL $mail] $sql] 
	update $this $sql
	
	# Max Session id :
	set sql "update Management \
	    set maxsid = (select max(sid) from Header where uid = $uid and mid = $mid)"
	update $this $sql

	# Management is up to date. Now, update MaxTable :
	
	update $this "delete from MaxTable"

	set sql {
	    insert into MaxTable
	    select type, max(id), 0
	    from Header 
	    where uid = UID 
	    group by type
	}

	set sql [string map [list UID $uid] $sql]
	update $this $sql

	# When some tables have no entry, load 0 as max.

	set sql {
	    insert into MaxTable
	    select classname, 0, 0
	    from ClassName
	    where classname not in (select type from MaxTable)
	}
	update $this $sql
	
    }
    

    proc fileExists {} {
	#puts "[getDBFilePath] existe : [file exists [getDBFilePath]]"
	return [file exists [getDBFilePath]]
    }
}