# seshuser.tcl --
#
#        basic information on user
#        SUBJECT TO CHANGE.
#
# Copyright (c) 1998,2003 Serge Rosmorduc
#

# We considered at a time saving user data in the database. However,
# we did realise that switching database, for instance, would 
# perhaps be a bit more difficult then.

# 
# Modification : the path is fixed in the case of non unix systems.
# 

package provide seshuser 0.0

package require legendes
package require TkseshDB
#package require fileentry

namespace eval seshuser {
    variable VARS

    namespace export getvar setvar
    namespace export dialogue loaduser
    namespace export cancel
    
    proc getvar {what} {
	variable VARS
	if [info exists VARS($what)] {
	    return $VARS($what) 
	} else {
	    global tcl_platform
	    global env
	    # Tries to get some reasonable defaults for the variables :
	    switch -- $what {
		NAME {return $tcl_platform(user)}
		ID {return "1234567890"}
		MID {return "0"}
		PATH {return [PRIVgetPath]}
		default {return ""}
	    }
	}
    }

    proc getUID {} {
	return [getvar ID]
    }

    proc getMID {} {
	return 0
    }

    proc setvar {what val} {
	variable VARS
	set VARS($what) $val
    }

    proc PRIVgetPath {} {
	global env
	if {[isUnix]} {
	    return [file join $env(HOME) Textes]
	} else {
	    return [file normalize [file join $env(SESHLIB) ..]]
	}
    }

    
    proc isUnix {} {
	return [expr {$::tcl_platform(platform) eq "unix"}]
    }

    proc isWindows {} {
	return [expr {$::tcl_platform(platform) eq "windows"}]
    }

    proc dialogue {} {
	variable VARS
	set w .userd
	toplevel $w
	wm title $w "User Information"
	label $w.expl -text [legendes::trans "Please enter your name, email, and ID below. \nTo get an ID and a password, please mail \n rosmord@iut.univ-paris8.fr. To try tksesh without ID, enter ID 1234567890"]
	label $w.name -text [legendes::trans "User Name"]
	entry $w.name_e
	label $w.code -text [legendes::trans "User ID"]
	entry $w.code_e

	label $w.mid -text [legendes::trans "Machine ID"]
	entry $w.mid_e

	label $w.email -text [legendes::trans "email"]
	entry $w.email_e
	label $w.lang -text [legendes::trans "language"]
	
	menubutton $w.lang_e -menu $w.lang_e.m -textvariable seshuser::VARS(language)\
	    -relief sunken -width 19 -font [$w.code_e cget -font] -justify left\
	    -takefocus 1 -highlightthickness [$w.code_e cget -highlightthickness]
	
	menu $w.lang_e.m -tearoff 0 
	
	foreach lang [legendes::listlanguages] {
	    $w.lang_e.m add radiobutton -label $lang -variable seshuser::VARS(language)
	}
	
	if {[isUnix]} {
	    label $w.path -text [legendes::trans "texts directory"]
	    frame $w.dirFrame
	    entry $w.dirFrame.path_e -textvariable seshuser::tmpdirname
	    button $w.dirFrame.browse -text [legendes::trans "browse"]\
		-command [list seshuser::chooseDir $w]
	}
	
	button $w.ok -text  [legendes::trans "ok"] -command seshuser::save_info 
	button $w.cancel -text  [legendes::trans "cancel"] -command [list seshuser::cancel $w]
	catch {
	    $w.name_e insert insert [getvar NAME]
	    $w.code_e insert insert [getvar ID]
	    $w.mid_e insert insert [getvar MID]
	    $w.email_e insert insert [getvar MAIL]
	    if {[isUnix]} {
		set seshuser::tmpdirname [getvar PATH]
	    }
	    set VARS(language) [getvar LANG]
	    if {[string length $VARS(language)] == 0} {
		set VARS(language) "english"
	    }
	}

	grid $w.expl -
	grid $w.name $w.name_e
	grid $w.code $w.code_e	
	grid $w.mid $w.mid_e	
	grid $w.email $w.email_e
	grid $w.lang $w.lang_e
	if {[isUnix]} {
	    grid $w.path $w.dirFrame
	    grid $w.dirFrame.path_e  $w.dirFrame.browse
	}
	grid $w.ok $w.cancel
	tkwait window $w
    }

    proc cancel {w} {
	variable VARS
	set lang $VARS(language)
	setvar LANG $lang
	catch {legendes::set$lang}
	destroy $w
    }

    # save user information,
    # ensuring everything is fine.
    
    proc save_info {} {
	variable VARS
	
	set w .userd
	set nom [$w.name_e get] 
	set id [$w.code_e get] 
	set mid [$w.mid_e get] 
	if {! [string is integer -strict $mid]} {
	    set mid 0
	    $w.mid_e delete 0 end
	    $w.mid_e insert insert "0"
	}
	set mail [$w.email_e get] 

	if {[isUnix]} {
	    set path [$w.dirFrame.path_e get] 
	} else {
	    set path [getvar PATH]
	}

	set lang $VARS(language)
	setvar LANG $lang
	catch {legendes::set$lang}
	
	if {[string compare $nom ""] != 0 && 
	    [string compare $id ""] != 0} {
	    setvar ID $id
	    setvar MID $mid
	    setvar NAME $nom
	    setvar MAIL $mail
	    if {[isUnix]} {
		setvar PATH $path
	    }
	    if [ensure_exists $path] {
		writeUser
		destroy $w
	    }
	}
    }

    # Load user informations
    # this will depend on the actual system.
    # for mac, we will use PREF_FOLDER to store the file
    # for windows, we will use the registry !
    #
    # return -- 1 if loaduser succeeded, else 0
    
    proc loaduser {} {
	set fname [PRIVgetUserPreferenceFile]
	if [file exists $fname] {
	    setvar LANG english
	    source $fname
	    legendes::set[getvar LANG]
	    if {! [isDefined MID]} {
		return 0
	    }
	    return 1
	} else {
	    return 0
	}
    }


    proc isDefined {what} {
	variable VARS
	return [info exists VARS($what)]
    }

    proc superUser {} {
	global env
	setvar ID 0
	setvar MID 0
	setvar NAME "admin"
	setvar MAIL ""
	setvar LANG "english"
	setvar PATH $env(SESHLIB)
    }
    
    # Actual writing of user informations
    # the information is supposed to be correct.
    # 
    proc writeUser {} {
	variable VARS
	set fname [PRIVgetUserPreferenceFile]
	if [info exists VARS(ID) ] {
	    set f [open $fname w]
	    puts $f [list ::seshuser::setvar NAME $VARS(NAME)]
	    puts $f [list ::seshuser::setvar ID $VARS(ID)]
	    puts $f [list ::seshuser::setvar MID $VARS(MID)]
	    puts $f [list ::seshuser::setvar MAIL $VARS(MAIL)]
	    if {[isUnix]} {
		puts $f [list ::seshuser::setvar PATH $VARS(PATH)]
	    }
	    puts $f [list ::seshuser::setvar LANG $VARS(LANG)]
	    writeUserToDB
	    close $f
	}
    }

    proc writeUserToDB {} {
	variable VARS
	if {[TkseshDB::fileExists]} {
	    TkseshDB::begin [TkseshDB::getTkseshDB]
	    TkseshDB::saveUser [TkseshDB::getTkseshDB] \
		$VARS(ID) $VARS(MID) $VARS(NAME) $VARS(MAIL)
	    TkseshDB::commit [TkseshDB::getTkseshDB]
	}
    }

    proc ensure_exists {path} {
	if { ! [file exists $path] } {
	    set q [tk_dialog .dialog "confirmation" "directory $path does not exist.\n Do you want to create it ?" "" 0 "Yes" "No"]
	    if {$q == 0} {
		file mkdir $path
	    } else {
		return 0
	    }
	} else {
	    if { ! [file isdirectory $path] } {
		tk_dialog .dialog "warning" "$path is not a directory.\n You should perhaps choose another name." "" "OK"
		return 0
	    }
	}
	return 1
    }
    
    proc chooseDir {w} {
	variable tmpdirname
	set dir [tk_chooseDirectory -title [legendes::trans "Tksesh user directory"]]
	set seshuser::tmpdirname $dir
    }

    proc PRIVgetUserPreferenceFile {} {
	if {[isWindows]} {
		return [file join $::env(SESHLIB) .seshrc]
	} else {
	    	return [file join $::env(HOME) .seshrc]
	}
    }

    proc loadOrCreateUser {} {
      package require TkseshDB
	wm withdraw . 

	if {! [seshuser::loaduser] } {
    		seshuser::dialogue
	}

	if {! [seshuser::loaduser] } {
    		tk_messageBox -type ok -message [legendes::trans "You should have defined a user"]
    		exit 1
	}

	# Ensures the database is defined
	# Normally, the directory is supposed to exist.
	seshuser::ensure_exists [seshuser::getvar PATH]

	if { ! [file isdirectory [seshuser::getvar PATH]]} {
    		tk_messageBox -type ok -message [legendes::trans "I really need"][seshuser::getvar PATH]
    		exit 1
	}

	# Now, test if the file exists :
	if {! [file isfile [TkseshDB::getDBFilePath]]} {
    		package require dbmaker
    		dbmaker::installDB
    		seshuser::writeUserToDB
	}

	if {! [file isfile [TkseshDB::getDBFilePath]]} {
    		tk_messageBox -type ok -message \
			[concat [legendes::trans "I could not create the database"] \
	     		" " [TkseshDB::getDBFilePath]]
    		exit 1
	}
	wm deiconify .    
    }

}


