
package require tclSQLDB
package require myStoopWidgets

# A widget for editing a field in a DataSource
package provide tkDB 1.0

class BDBField {

    #
    # parent : the parent widget
    # name : name for this widget ("auto" lets the system name the widget)
    # args : various args to pass.
    #
    # dataSource : the DataSource the field belongs to
    # field : the field to display
    #
    
    proc BDBField {this parent name dataSource field args} SEntry {$parent $name 
	-validate key -validatecommand [list BDBField::doValidate $this %P] -invalidcommand bell $args} Observer {} {
	    set ($this,dataSource) $dataSource
	    set ($this,field) $field
	    # Refreshing is used to avoid circular dependencies
	    set ($this,refreshing) 0
	}
    
    proc ~BDBField {this} {
    }
    
    virtual proc update {this observable msg} {
	puts "updating  $this for $msg"
	switch -- $msg {
	    "ObservableDeleted" {}
	    "IndexChanged" -
	    "dataChanged" -
	    "rowDeleted" {doRefresh $this}
	    "rowChanged" {}
	    "rowCreated" {}
	}
    }
    
    # PRIVATE proc
    proc doRefresh {this} {
	if {! $($this,refreshing)} {
	    set ($this,refreshing) 1
	    set error [catch {
		Component::cset $this state normal
		Component::cset $this text [DataSource::getCol $($this,dataSource) $($this,field)] 
	    } errorMsg]
	    if {$error} {
		Component::cset $this state disabled
	    }
	    # fix the editable state :
	    set editable [DataSource::canDo $($this,dataSource) setCol $($this,field)]
	    Component::cset $this editable $editable
	    set ($this,refreshing) 0
	}
    }
    
    proc doValidate {this txt} {
	if {! $($this,refreshing)} {
	    set ($this,refreshing) 1
	    puts "validating $txt"
	    DataSource::setCol $($this,dataSource) $($this,field) $txt
	    set ($this,refreshing) 0
	}
	return 1
    }
    
    virtual proc completorHook {this} {
	Observable::add $($this,dataSource) $this
	doRefresh $this
    }

}

