
# A navigator for moving and controling a DataSource
package require icons

icons::doIcons

class BDBNavigator {
    
    # Note about the "buttons" variable : it allows to select only some of the buttons.
    # it's a list of 0 and 1. The first value is for "move" buttons.
    # the second is for delete
    # the third is for create
    # the fourth is for cancel/validate (mandatory if you want editing or creation functions)

    proc BDBNavigator {this parent name dataSource {buttons {1 1 1 1}} args} SFrame {$parent $name $args} \
	Observer {} {
	    set ($this,dataSource) $dataSource
	    foreach {move delete create cancel} $buttons {
		set ($this,hasMove) $move
		set ($this,hasDelete) $delete
		set ($this,hasCreate) $create
		set ($this,hasCancel) $cancel
	    }
	}

    
    proc ~BDBNavigator {this} {
	catch {
	    # Unregister :
	    Observable::remove $($this,dataSource) $this
	}
    }

    virtual proc update {this observable msg} {
	puts "getting a $msg"
	switch -- $msg {
	    "ObservableDeleted" {}
	    "stateChanged" -
	    "IndexChanged" -
	    "dataChanged" -
	    "rowDeleted" -
	    "rowChanged" -
	    "rowCreated" {
		refresh $this
	    }
	}
    }

    virtual proc completorHook {this} {
	set w [Component::getPath $this]
	set blist {}
	
	if {$($this,hasMove)} {
	    Button $w.previous -command [list BDBNavigator::previous $this] -image ::icon::back\
		-helptext "move back one row" -helptype balloon
	    Button $w.next -command [list BDBNavigator::next $this] -image ::icon::forward\
		-helptext "move forward one row" -helptype balloon	    
	    lappend blist $w.previous $w.next
	}
	if {$($this,hasDelete)} {
	    Button $w.delete -command [list BDBNavigator::delete $this] -image ::icon::edit_remove
	    lappend blist $w.delete
	}
	if {$($this,hasCreate)} {
	    Button $w.new -command [list BDBNavigator::create $this] -image ::icon::edit_add \
		-helptext "create a new row" -helptype balloon
	    lappend blist $w.new
	}
	if {$($this,hasCancel)} {
	    Button $w.cancel -command [list BDBNavigator::cancel $this] -image ::icon::undo \
		-helptext "cancel current editing" -helptype balloon
	    Button $w.validate -command [list BDBNavigator::validate $this] -image ::icon::ok \
		-helptext "validate current editing"
	    lappend blist $w.cancel $w.validate 
	}
	eval grid $blist
	# grid $w.previous $w.next $w.delete $w.new 
	Observable::add $($this,dataSource) $this
	refresh $this
    }

    proc refresh {this} {
	set w [Component::getPath $this]
	foreach {bname comm} [list \
				  $w.next next\
				  $w.previous previous\
				  $w.cancel cancel\
				  $w.validate validate\
				  $w.delete deleteRow\
				  $w.new createRow ] {
	    catch {
		if {[DataSource::canDo $($this,dataSource) $comm]} {
		    $bname configure -state normal
		} else {
		    $bname configure -state disabled
		}
	    }
	}
    }
    proc next {this} {
	DataSource::next $($this,dataSource)
    }
    
    proc previous {this} {
	DataSource::previous $($this,dataSource)
    }

    proc validate {this} {
	DataSource::validate $($this,dataSource)
    }
    
    proc create {this} {
	DataSource::createRow $($this,dataSource)
    }
    
    proc delete {this} {
	DataSource::deleteRow $($this,dataSource)
    }

    proc cancel {this} {
	DataSource::cancel $($this,dataSource)
    }
}
